@@ -78,6 +78,36 @@ katze_widget_popup                   (GtkWidget*      widget,
 GtkWidget*
 katze_image_menu_item_new_ellipsized (const gchar*   label);
 
+GdkPixbuf*
+katze_pixbuf_new_from_buffer         (const guchar* buffer,
+                                      gsize         length,
+                                      const gchar*  mime_type,
+                                      GError**      error);
+
+gint
+katze_object_get_boolean             (gpointer     object,
+                                      const gchar* property);
+
+gint
+katze_object_get_int                 (gpointer     object,
+                                      const gchar* property);
+
+gfloat
+katze_object_get_float               (gpointer     object,
+                                      const gchar* property);
+
+gint
+katze_object_get_enum                (gpointer     object,
+                                      const gchar* property);
+
+gchar*
+katze_object_get_string              (gpointer     object,
+                                      const gchar* property);
+
+gpointer
+katze_object_get_object              (gpointer     object,
+                                      const gchar* property);
+
 G_END_DECLS
 
 #endif /* __KATZE_UTILS_H__ */
