@@ -14,11 +14,16 @@
     #include <config.h>
 #endif
 
-#include "midori-view.h"
+#include "midori-addons.h"
 #include "midori-app.h"
-#include "midori-websettings.h"
 #include "midori-browser.h"
+#include "midori-console.h"
+#include "midori-extension.h"
+#include "midori-panel.h"
 #include "midori-stock.h"
+#include "midori-view.h"
+#include "midori-websettings.h"
+
 
 #include "sokoke.h"
 #include "gjs.h"
@@ -26,13 +31,14 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libxml/parser.h>
-#include <libxml/tree.h>
 
-#if HAVE_SQLITE3_H
+#ifdef HAVE_LIBXML
+    #include <libxml/parser.h>
+    #include <libxml/tree.h>
+#endif
+
+#ifdef HAVE_SQLITE
     #include <sqlite3.h>
-#elif HAVE_SQLITE_H
-    #include <sqlite.h>
 #endif
 
 #if ENABLE_NLS
@@ -59,6 +65,10 @@ stock_items_init (void)
         guint keyval;
         gchar* fallback;
     } FatStockItem;
+    GtkIconSource* icon_source;
+    GtkIconSet* icon_set;
+    GtkIconFactory* factory = gtk_icon_factory_new ();
+    gsize i;
 
     static FatStockItem items[] =
     {
@@ -72,33 +82,18 @@ stock_items_init (void)
         { STOCK_BOOKMARKS,      N_("_Bookmarks"), 0, 0, GTK_STOCK_DIRECTORY },
         { STOCK_BOOKMARK_ADD,   N_("_Add Bookmark"), 0, 0, GTK_STOCK_ADD },
         { STOCK_CONSOLE,        N_("_Console"), 0, 0, GTK_STOCK_DIALOG_WARNING },
-        { STOCK_DISABLE,        N_("_Disable"), 0, 0, GTK_STOCK_NO },
-        { STOCK_ENABLE,         N_("_Enable"), 0, 0, GTK_STOCK_YES },
         { STOCK_EXTENSIONS,     N_("_Extensions"), 0, 0, GTK_STOCK_CONVERT },
         { STOCK_HISTORY,        N_("_History"), 0, 0, GTK_STOCK_SORT_ASCENDING },
         { STOCK_HOMEPAGE,       N_("_Homepage"), 0, 0, GTK_STOCK_HOME },
-        { STOCK_PAGE_HOLDER,    N_("_Pageholder"), 0, 0, GTK_STOCK_ORIENTATION_PORTRAIT },
         { STOCK_SCRIPTS,        N_("_Userscripts"), 0, 0, GTK_STOCK_EXECUTE },
         { STOCK_STYLES,         N_("User_styles"), 0, 0, GTK_STOCK_SELECT_COLOR },
         { STOCK_TAB_NEW,        N_("New _Tab"), 0, 0, GTK_STOCK_ADD },
         { STOCK_TRANSFERS,      N_("_Transfers"), 0, 0, GTK_STOCK_SAVE },
         { STOCK_USER_TRASH,     N_("_Closed Tabs and Windows"), 0, 0, "gtk-undo-ltr" },
         { STOCK_WINDOW_NEW,     N_("New _Window"), 0, 0, GTK_STOCK_ADD },
-        #if !GTK_CHECK_VERSION(2, 10, 0)
-        { GTK_STOCK_SELECT_ALL, N_("Select _All"), 0, 0, NULL },
-        #endif
-        #if !GTK_CHECK_VERSION(2, 8, 0)
-        { GTK_STOCK_FULLSCREEN, N_("_Fullscreen"), 0, 0, NULL },
-        { GTK_STOCK_LEAVE_FULLSCREEN, N_("_Leave Fullscreen"), 0, 0, NULL },
-        #endif
     };
 
-    GtkIconSource* icon_source;
-    GtkIconSet* icon_set;
-    GtkIconFactory* factory = gtk_icon_factory_new ();
-    guint i;
-
-    for (i = 0; i < (guint)G_N_ELEMENTS (items); i++)
+    for (i = 0; i < G_N_ELEMENTS (items); i++)
     {
         icon_set = gtk_icon_set_new ();
         icon_source = gtk_icon_source_new ();
@@ -140,7 +135,7 @@ settings_new_from_file (const gchar* filename)
                                    G_KEY_FILE_KEEP_COMMENTS, &error))
     {
         if (error->code != G_FILE_ERROR_NOENT)
-            printf (_("The configuration couldn't be loaded. %s\n"),
+            printf (_("The configuration couldn't be loaded: %s\n"),
                     error->message);
         g_error_free (error);
     }
@@ -202,7 +197,7 @@ settings_new_from_file (const gchar* filename)
             g_type_class_unref (enum_class);
         }
         else
-            g_warning (_("Unhandled settings value '%s'"), property);
+            g_warning (_("Invalid configuration value '%s'"), property);
     }
     return settings;
 }
@@ -219,6 +214,7 @@ settings_save_to_file (MidoriWebSettings* settings,
     GParamSpec* pspec;
     GType type;
     const gchar* property;
+    gboolean saved;
 
     key_file = g_key_file_new ();
     class = G_OBJECT_GET_CLASS (settings);
@@ -271,9 +267,9 @@ settings_save_to_file (MidoriWebSettings* settings,
                                    enum_value->value_name);
         }
         else
-            g_warning (_("Unhandled settings value '%s'"), property);
+            g_warning (_("Invalid configuration value '%s'"), property);
     }
-    gboolean saved = sokoke_key_file_save_to_file (key_file, filename, error);
+    saved = sokoke_key_file_save_to_file (key_file, filename, error);
     g_key_file_free (key_file);
     return saved;
 }
@@ -374,6 +370,7 @@ midori_web_list_add_item_cb (KatzeArray* trash,
     }
 }
 
+#ifdef HAVE_LIBXML
 static KatzeItem*
 katze_item_from_xmlNodePtr (xmlNodePtr cur)
 {
@@ -513,39 +510,6 @@ katze_array_from_xmlDocPtr (KatzeArray* array,
     return TRUE;
 }
 
-#if 0
-static gboolean
-katze_array_from_data (KatzeArray*  array,
-                       const gchar* data,
-                       GError**     error)
-{
-    xmlDocPtr doc;
-
-    g_return_val_if_fail (katze_array_is_a (array, KATZE_TYPE_ITEM), FALSE);
-    g_return_val_if_fail (data != NULL, FALSE);
-
-    katze_array_clear (array);
-
-    if((doc = xmlParseMemory (data, strlen (data))) == NULL)
-    {
-        /* No valid xml or broken encoding */
-        *error = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                                      _("Malformed document."));
-        return FALSE;
-    }
-    if (!katze_array_from_xmlDocPtr (array, doc))
-    {
-        /* Parsing failed */
-        xmlFreeDoc (doc);
-        *error = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                                      _("Malformed document."));
-        return FALSE;
-    }
-    xmlFreeDoc (doc);
-    return TRUE;
-}
-#endif
-
 static gboolean
 katze_array_from_file (KatzeArray*  array,
                        const gchar* filename,
@@ -583,6 +547,7 @@ katze_array_from_file (KatzeArray*  array,
     xmlFreeDoc (doc);
     return TRUE;
 }
+#endif
 
 #ifdef HAVE_SQLITE
 /* Open database 'dbname' */
@@ -598,7 +563,7 @@ db_open (const char* dbname,
         {
             *error = g_error_new (MIDORI_HISTORY_ERROR,
                                   MIDORI_HISTORY_ERROR_DB_OPEN,
-                                  _("Error opening database: %s\n"),
+                                  _("Failed to open database: %s\n"),
                                   sqlite3_errmsg (db));
         }
         sqlite3_close (db);
@@ -630,7 +595,7 @@ db_exec_callback (sqlite3*    db,
         {
             *error = g_error_new (MIDORI_HISTORY_ERROR,
                                   MIDORI_HISTORY_ERROR_EXEC_SQL,
-                                  _("Error opening database: %s\n"),
+                                  _("Failed to open database: %s\n"),
                                   errmsg);
         }
         sqlite3_free (errmsg);
@@ -677,15 +642,14 @@ midori_history_remove_item_cb (KatzeArray* history,
 
     sqlcmd = g_strdup_printf (
         "DELETE FROM history WHERE uri = '%s' AND"
-        " title = '%s' AND date = %" G_GINT64_FORMAT " AND visits = %d",
+        " title = '%s' AND date = %" G_GINT64_FORMAT,
         katze_item_get_uri (item),
         katze_item_get_name (item),
-        katze_item_get_added (item),
-        katze_item_get_visits (item));
+        katze_item_get_added (item));
     success = db_exec (db, sqlcmd, &error);
     if (!success)
     {
-        g_printerr (_("Failed to remove history item. %s\n"), error->message);
+        g_printerr (_("Failed to remove history item: %s\n"), error->message);
         g_error_free (error);
         return ;
     }
@@ -709,7 +673,7 @@ midori_history_clear_cb (KatzeArray* history,
 
     if (!db_exec (db, "DELETE FROM history", &error))
     {
-        g_printerr (_("Failed to clear history. %s\n"), error->message);
+        g_printerr (_("Failed to clear history: %s\n"), error->message);
         g_error_free (error);
     }
 }
@@ -742,22 +706,21 @@ midori_history_add_item_cb (KatzeArray* array,
         if (!db_exec_callback (db, "SELECT date('now')",
                                gettimestr, item, &error))
         {
-            g_printerr (_("Failed to add history item. %s\n"), error->message);
+            g_printerr (_("Failed to add history item: %s\n"), error->message);
             g_error_free (error);
             return;
         }
     }
     sqlcmd = g_strdup_printf ("INSERT INTO history VALUES"
-                              "('%s', '%s', %" G_GUINT64_FORMAT ", %d)",
+                              "('%s', '%s', %" G_GUINT64_FORMAT ", -1)",
                               katze_item_get_uri (item),
                               katze_item_get_name (item),
-                              katze_item_get_added (item),
-                              katze_item_get_visits (item));
+                              katze_item_get_added (item));
     success = db_exec (db, sqlcmd, &error);
     g_free (sqlcmd);
     if (!success)
     {
-        g_printerr (_("Failed to add history item. %s\n"), error->message);
+        g_printerr (_("Failed to add history item: %s\n"), error->message);
         g_error_free (error);
         return ;
     }
@@ -775,7 +738,7 @@ midori_history_add_items (void*  data,
     gint64 date;
     time_t newdate;
     gint i, j, n;
-    gint ncols = 4;
+    gint ncols = 3;
 
     g_return_val_if_fail (KATZE_IS_ARRAY (array), 1);
 
@@ -788,22 +751,20 @@ midori_history_add_items (void*  data,
         {
             if (colname[i] && !g_ascii_strcasecmp (colname[i], "uri") &&
                 colname[i + 1] && !g_ascii_strcasecmp (colname[i + 1], "title") &&
-                colname[i + 2] && !g_ascii_strcasecmp (colname[i + 2], "date") &&
-                colname[i + 3] && !g_ascii_strcasecmp (colname[i + 3], "visits"))
+                colname[i + 2] && !g_ascii_strcasecmp (colname[i + 2], "date"))
             {
                 item = katze_item_new ();
                 katze_item_set_uri (item, argv[i]);
                 katze_item_set_name (item, argv[i + 1]);
                 date = g_ascii_strtoull (argv[i + 2], NULL, 10);
                 katze_item_set_added (item, date);
-                katze_item_set_visits (item, atoi (argv[i + 3]));
 
                 n = katze_array_get_length (array);
                 for (j = 0; j < n; j++)
                 {
                     parent = katze_array_get_nth_item (array, j);
                     newdate = katze_item_get_added (KATZE_ITEM (parent));
-                    if (sokoke_same_day (&date, &newdate))
+                    if (sokoke_same_day ((time_t *)&date, (time_t *)&newdate))
                         break;
                 }
                 if (j == n)
@@ -838,7 +799,7 @@ midori_history_initialize (KatzeArray*  array,
         return NULL;
 
     if (!db_exec_callback (db,
-                           "SELECT uri, title, date, visits FROM history "
+                           "SELECT uri, title, date FROM history "
                            "ORDER BY date ASC",
                            midori_history_add_items,
                            array,
@@ -874,7 +835,8 @@ midori_history_terminate (sqlite3* db,
     db_exec (db, sqlcmd, &error);
     if (!success)
     {
-        g_printerr (_("Failed to remove old history items. %s\n"), error->message);
+        /* i18n: Couldn't remove items that are older than n days */
+        g_printerr (_("Failed to remove old history items: %s\n"), error->message);
         g_error_free (error);
         return ;
     }
@@ -1015,6 +977,81 @@ katze_array_to_file (KatzeArray*  array,
     return TRUE;
 }
 
+static void
+midori_view_console_message_cb (GtkWidget*     view,
+                                const gchar*   message,
+                                gint           line,
+                                const gchar*   source_id,
+                                MidoriConsole* console)
+{
+    midori_console_add (console, message, line, source_id);
+}
+
+static void
+midori_browser_add_tab_cb (MidoriBrowser* browser,
+                           MidoriView*    view,
+                           MidoriConsole* console)
+{
+    g_signal_connect (view, "console-message",
+        G_CALLBACK (midori_view_console_message_cb), console);
+}
+
+static void
+midori_app_add_browser_cb (MidoriApp*     app,
+                           MidoriBrowser* browser,
+                           KatzeNet*      net)
+{
+    GtkWidget* panel;
+    GtkWidget* addon;
+    GtkWidget* toolbar;
+
+    panel = katze_object_get_object (browser, "panel");
+
+    /* Transfers */
+    #if 0
+    addon = midori_view_new (net);
+    gtk_widget_show (addon);
+    midori_panel_append_widget (MIDORI_PANEL (panel), addon,
+                                STOCK_TRANSFERS, _("Transfers"), NULL);
+    #endif
+
+    /* Console */
+    addon = midori_console_new ();
+    gtk_widget_show (addon);
+    toolbar = midori_console_get_toolbar (MIDORI_CONSOLE (addon));
+    gtk_widget_show (toolbar);
+    midori_panel_append_widget (MIDORI_PANEL (panel), addon,
+                                STOCK_CONSOLE, _("Console"), toolbar);
+    g_signal_connect (browser, "add-tab",
+        G_CALLBACK (midori_browser_add_tab_cb), addon);
+
+    /* Userscripts */
+    addon = midori_addons_new (MIDORI_ADDON_USER_SCRIPTS, GTK_WIDGET (browser));
+    gtk_widget_show (addon);
+    toolbar = midori_addons_get_toolbar (MIDORI_ADDONS (addon));
+    gtk_widget_show (toolbar);
+    midori_panel_append_widget (MIDORI_PANEL (panel), addon,
+                                STOCK_SCRIPTS, _("Userscripts"), toolbar);
+
+    /* Userstyles */
+    addon = midori_addons_new (MIDORI_ADDON_USER_STYLES, GTK_WIDGET (browser));
+    gtk_widget_show (addon);
+    toolbar = midori_addons_get_toolbar (MIDORI_ADDONS (addon));
+    gtk_widget_show (toolbar);
+    midori_panel_append_widget (MIDORI_PANEL (panel), addon,
+                                STOCK_STYLES, _("Userstyles"), toolbar);
+
+    /* Extensions */
+    #if 0
+    addon = midori_addons_new (MIDORI_ADDON_EXTENSIONS);
+    gtk_widget_show (addon);
+    toolbar = midori_addons_get_toolbar (MIDORI_ADDONS (addon));
+    gtk_widget_show (toolbar);
+    midori_panel_append_page (MIDORI_PANEL (panel), addon,
+                              STOCK_EXTENSIONS, _("Extensions"), toolbar);
+    #endif
+}
+
 static void
 midori_browser_session_cb (MidoriBrowser* browser,
                            gpointer       arg1,
@@ -1026,7 +1063,7 @@ midori_browser_session_cb (MidoriBrowser* browser,
 
     config_path = g_build_filename (g_get_user_config_dir (),
                                     PACKAGE_NAME, NULL);
-    g_mkdir_with_parents (config_path, 0755);
+    g_mkdir_with_parents (config_path, 0700);
     config_file = g_build_filename (config_path, "session.xbel", NULL);
     error = NULL;
     if (!katze_array_to_file (session, config_file, &error))
@@ -1047,9 +1084,6 @@ midori_browser_weak_notify_cb (MidoriBrowser* browser,
                          G_CALLBACK (midori_browser_session_cb), session, NULL);
 }
 
-gboolean
-midori_view_single_process (gboolean enable);
-
 int
 main (int    argc,
       char** argv)
@@ -1064,7 +1098,7 @@ main (int    argc,
     {
        { "run", 'r', 0, G_OPTION_ARG_NONE, &run,
        N_("Run the specified filename as javascript"), NULL },
-       { "version", 'v', 0, G_OPTION_ARG_NONE, &version,
+       { "version", 'V', 0, G_OPTION_ARG_NONE, &version,
        N_("Display program version"), NULL },
        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &uris,
        N_("URIs"), NULL },
@@ -1091,7 +1125,7 @@ main (int    argc,
     #endif
 
     #if ENABLE_NLS
-    bindtextdomain (GETTEXT_PACKAGE, MIDORI_LOCALEDIR);
+    bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
     #endif
@@ -1109,6 +1143,9 @@ main (int    argc,
         return 1;
     }
 
+    /* libSoup uses threads, therefore if WebKit is built with libSoup
+       or Midori is using it, we need to initialize threads. */
+    if (!g_thread_supported ()) g_thread_init (NULL);
     stock_items_init ();
     g_set_application_name (_("Midori"));
 
@@ -1162,8 +1199,8 @@ main (int    argc,
         if (result)
             return 0;
 
-        /* FIXME: Do we want a graphical error message? */
         g_print (_("An instance of Midori is already running but not responding.\n"));
+        /* FIXME: Do we want a graphical error message? */
         return 1;
     }
 
@@ -1171,7 +1208,7 @@ main (int    argc,
     GString* error_messages = g_string_new (NULL);
     gchar* config_path = g_build_filename (g_get_user_config_dir (),
                                            PACKAGE_NAME, NULL);
-    g_mkdir_with_parents (config_path, 0755);
+    g_mkdir_with_parents (config_path, 0700);
     gchar* config_file = g_build_filename (config_path, "config", NULL);
     error = NULL;
     MidoriWebSettings* settings = settings_new_from_file (config_file);
@@ -1189,19 +1226,22 @@ main (int    argc,
                 error->message); */
         g_error_free (error);
     }
+    bookmarks = katze_array_new (KATZE_TYPE_ARRAY);
+    #ifdef HAVE_LIBXML
     katze_assign (config_file, g_build_filename (config_path, "bookmarks.xbel",
                                                  NULL));
-    bookmarks = katze_array_new (KATZE_TYPE_ARRAY);
     error = NULL;
     if (!katze_array_from_file (bookmarks, config_file, &error))
     {
         if (error->code != G_FILE_ERROR_NOENT)
             g_string_append_printf (error_messages,
-                _("The bookmarks couldn't be loaded. %s\n"), error->message);
+                _("The bookmarks couldn't be loaded: %s\n"), error->message);
         g_error_free (error);
     }
     g_free (config_file);
+    #endif
     _session = katze_array_new (KATZE_TYPE_ITEM);
+    #ifdef HAVE_LIBXML
     g_object_get (settings, "load-on-startup", &load_on_startup, NULL);
     if (load_on_startup == MIDORI_STARTUP_LAST_OPEN_PAGES)
     {
@@ -1211,22 +1251,25 @@ main (int    argc,
         {
             if (error->code != G_FILE_ERROR_NOENT)
                 g_string_append_printf (error_messages,
-                    _("The session couldn't be loaded. %s\n"), error->message);
+                    _("The session couldn't be loaded: %s\n"), error->message);
             g_error_free (error);
         }
         g_free (config_file);
     }
-    config_file = g_build_filename (config_path, "tabtrash.xbel", NULL);
+    #endif
     trash = katze_array_new (KATZE_TYPE_ITEM);
+    #ifdef HAVE_LIBXML
+    config_file = g_build_filename (config_path, "tabtrash.xbel", NULL);
     error = NULL;
     if (!katze_array_from_file (trash, config_file, &error))
     {
         if (error->code != G_FILE_ERROR_NOENT)
             g_string_append_printf (error_messages,
-                _("The trash couldn't be loaded. %s\n"), error->message);
+                _("The trash couldn't be loaded: %s\n"), error->message);
         g_error_free (error);
     }
     g_free (config_file);
+    #endif
     #ifdef HAVE_SQLITE
     config_file = g_build_filename (config_path, "history.db", NULL);
     #endif
@@ -1236,7 +1279,7 @@ main (int    argc,
     if ((db = midori_history_initialize (history, config_file, &error)) == NULL)
     {
         g_string_append_printf (error_messages,
-            _("The history couldn't be loaded. %s\n"), error->message);
+            _("The history couldn't be loaded: %s\n"), error->message);
         g_error_free (error);
     }
     g_free (config_file);
@@ -1265,7 +1308,7 @@ main (int    argc,
             GTK_MESSAGE_DIALOG (dialog), "%s", error_messages->str);
         gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                "_Ignore", GTK_RESPONSE_ACCEPT,
+                                _("_Ignore"), GTK_RESPONSE_ACCEPT,
                                 NULL);
         if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_ACCEPT)
         {
@@ -1326,12 +1369,65 @@ main (int    argc,
         G_CALLBACK (midori_history_clear_cb), db);
     #endif
 
+    /* Load extensions */
+    KatzeArray* extensions;
+    gchar* extension_path;
+    GDir* extension_dir;
+    const gchar* filename;
+    MidoriExtension* extension;
+
+    extensions = katze_array_new (MIDORI_TYPE_EXTENSION);
+    extension_path = g_build_filename (LIBDIR, PACKAGE_NAME, NULL);
+    if (g_module_supported ())
+        extension_dir = g_dir_open (extension_path, 0, NULL);
+    else
+        extension_dir = NULL;
+    if (extension_dir)
+    {
+        while ((filename = g_dir_read_name (extension_dir)))
+        {
+            gchar* fullname;
+            GModule* module;
+            typedef MidoriExtension* (*extension_init_func)(void);
+            extension_init_func extension_init;
+
+            fullname = g_build_filename (extension_path, filename, NULL);
+            module = g_module_open (fullname, G_MODULE_BIND_LOCAL);
+            g_free (fullname);
+            if (!module)
+            {
+                g_warning ("%s", g_module_error ());
+                continue;
+            }
+            ;
+            if (!g_module_symbol (module, "extension_init",
+                             (gpointer) &extension_init))
+            {
+                g_warning ("%s", g_module_error ());
+                continue;
+            }
+            extension = extension_init ();
+            katze_array_add_item (extensions, extension);
+        }
+        g_dir_close (extension_dir);
+    }
+
     g_object_set (app, "settings", settings,
                        "bookmarks", bookmarks,
                        "trash", trash,
                        "search-engines", search_engines,
                        "history", history,
+                       "extensions", extensions,
                        NULL);
+    g_signal_connect (app, "add-browser",
+        G_CALLBACK (midori_app_add_browser_cb), NULL);
+
+    n = katze_array_get_length (extensions);
+    for (i = 0; i < n; i++)
+    {
+        extension = katze_array_get_nth_item (extensions, i);
+        g_signal_emit_by_name (extension, "activate", app);
+    }
 
     browser = g_object_new (MIDORI_TYPE_BROWSER,
                             "settings", settings,
@@ -1365,40 +1461,12 @@ main (int    argc,
     g_object_weak_ref (G_OBJECT (session),
         (GWeakNotify)(midori_browser_weak_notify_cb), browser);
 
-    /* Load extensions */
-    js_context = gjs_global_context_new ();
-    /* FIXME: We want to honor system installed addons as well */
-    gchar* addon_path = g_build_filename (g_get_user_data_dir (), PACKAGE_NAME,
-                                          "extensions", NULL);
-    GDir* addon_dir = g_dir_open (addon_path, 0, NULL);
-    if (addon_dir)
-    {
-        const gchar* filename;
-        while ((filename = g_dir_read_name (addon_dir)))
-        {
-            if (!g_str_has_prefix (filename, ".midori.js"))
-                continue;
-
-            gchar* fullname = g_build_filename (addon_path, filename, NULL);
-            gchar* exception = NULL;
-            gjs_script_from_file (js_context, fullname, &exception);
-            if (exception)
-            /* FIXME: Do we want to print this somewhere else? */
-            /* FIXME Convert the filename to UTF8 */
-                printf ("%s - Exception: %s\n", filename, exception);
-            g_free (fullname);
-        }
-        g_dir_close (addon_dir);
-    }
-
     gtk_main ();
 
-    JSGlobalContextRelease (js_context);
-
     /* Save configuration files */
     config_path = g_build_filename (g_get_user_config_dir (), PACKAGE_NAME,
                                     NULL);
-    g_mkdir_with_parents (config_path, 0755);
+    g_mkdir_with_parents (config_path, 0700);
     g_object_unref (history);
     #ifdef HAVE_SQLITE
     g_object_get (settings, "maximum-history-age", &max_history_age, NULL);
