@@ -24,7 +24,7 @@
 #include <JavaScriptCore/JavaScript.h>
 #include <glib/gi18n.h>
 #include <string.h>
-#if HAVE_GIO
+#if GLIB_CHECK_VERSION (2, 16, 0)
     #include <gio/gio.h>
 #endif
 
@@ -32,8 +32,8 @@ struct _MidoriAddons
 {
     GtkVBox parent_instance;
 
-    GtkWidget* web_widget;
     MidoriAddonKind kind;
+    GtkWidget* web_widget;
     GtkWidget* toolbar;
     GtkWidget* treeview;
 
@@ -58,8 +58,8 @@ enum
 {
     PROP_0,
 
-    PROP_WEB_WIDGET,
-    PROP_KIND
+    PROP_KIND,
+    PROP_WEB_WIDGET
 };
 
 static void
@@ -85,7 +85,6 @@ midori_addon_kind_get_type (void)
     {
         static const GEnumValue values[] = {
          { MIDORI_ADDON_NONE, "MIDORI_ADDON_NONE", N_("None") },
-         { MIDORI_ADDON_EXTENSIONS, "MIDORI_ADDON_EXTENSIONS", N_("Extensions") },
          { MIDORI_ADDON_USER_SCRIPTS, "MIDORI_USER_SCRIPTS", N_("Userscripts") },
          { MIDORI_ADDON_USER_STYLES, "MIDORI_USER_STYLES", N_("Userstyles") },
          { 0, NULL, NULL }
@@ -108,15 +107,6 @@ midori_addons_class_init (MidoriAddonsClass* class)
 
     flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_WEB_WIDGET,
-                                     g_param_spec_object (
-                                     "web-widget",
-                                     "Web Widget",
-                                     "The assigned web widget",
-                                     GTK_TYPE_WIDGET,
-                                     G_PARAM_READWRITE));
-
     g_object_class_install_property (gobject_class,
                                      PROP_KIND,
                                      g_param_spec_enum (
@@ -126,6 +116,15 @@ midori_addons_class_init (MidoriAddonsClass* class)
                                      MIDORI_TYPE_ADDON_KIND,
                                      MIDORI_ADDON_NONE,
                                      flags));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_WEB_WIDGET,
+                                     g_param_spec_object (
+                                     "web-widget",
+                                     "Web Widget",
+                                     "The assigned web widget",
+                                     GTK_TYPE_WIDGET,
+                                     G_PARAM_READWRITE));
 }
 
 static void
@@ -138,11 +137,11 @@ midori_addons_set_property (GObject*      object,
 
     switch (prop_id)
     {
-    case PROP_WEB_WIDGET:
-        midori_addons_set_web_widget (addons, g_value_get_object (value));
-        break;
     case PROP_KIND:
-        midori_addons_set_kind (addons, g_value_get_enum (value));
+        addons->kind = g_value_get_enum (value);
+        break;
+    case PROP_WEB_WIDGET:
+        katze_object_assign (addons->web_widget, g_value_dup_object (value));
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -160,11 +159,11 @@ midori_addons_get_property (GObject*    object,
 
     switch (prop_id)
     {
-    case PROP_WEB_WIDGET:
-        g_value_set_object (value, midori_addons_get_web_widget (addons));
-        break;
     case PROP_KIND:
-        g_value_set_enum (value, midori_addons_get_kind (addons));
+        g_value_set_enum (value, addons->kind);
+        break;
+    case PROP_WEB_WIDGET:
+        g_value_set_object (value, addons->web_widget);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -177,8 +176,6 @@ _addons_get_folder (MidoriAddons* addons)
 {
     switch (addons->kind)
     {
-    case MIDORI_ADDON_EXTENSIONS:
-        return "extensions";
     case MIDORI_ADDON_USER_SCRIPTS:
         return "scripts";
     case MIDORI_ADDON_USER_STYLES:
@@ -193,8 +190,6 @@ _addons_get_extension (MidoriAddons* addons)
 {
     switch (addons->kind)
     {
-    case MIDORI_ADDON_EXTENSIONS:
-        return ".midori.js";
     case MIDORI_ADDON_USER_SCRIPTS:
         return ".user.js";
     case MIDORI_ADDON_USER_STYLES:
@@ -313,7 +308,7 @@ _addons_toggle_disable_button (MidoriAddons* addons,
     gtk_widget_set_sensitive (GTK_WIDGET (button), sensitive);
 }
 
-#if HAVE_GIO
+#if GLIB_CHECK_VERSION (2, 16, 0)
 static void
 midori_addons_directory_monitor_changed (GFileMonitor*     monitor,
                                          GFile*            child,
@@ -398,7 +393,7 @@ midori_addons_button_add_clicked_cb (GtkToolItem*  toolitem,
         _addons_get_folder (addons));
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
-    #if !HAVE_GIO
+    #if GLIB_CHECK_VERSION (2, 16, 0)
     /* FIXME: Without GIO clicking this button is the only
               way to update the list */
     midori_addons_update_elements (addons);
@@ -491,6 +486,9 @@ static void
 midori_addons_finalize (GObject* object)
 {
     MidoriAddons* addons = MIDORI_ADDONS (object);
+
+    katze_object_assign (addons->web_widget, NULL);
+
     g_slist_free (addons->elements);
 }
 
@@ -815,11 +813,9 @@ _js_style_from_file (JSContextRef js_context,
 }
 
 static void
-midori_web_widget_window_object_cleared_cb (GtkWidget*         web_widget,
-                                            WebKitWebFrame*    web_frame,
-                                            JSGlobalContextRef js_context,
-                                            JSObjectRef        js_window,
-                                            MidoriAddons*      addons)
+midori_web_widget_context_ready_cb (GtkWidget*         web_widget,
+                                    JSGlobalContextRef js_context,
+                                    MidoriAddons*      addons)
 {
     const gchar* uri;
     GSList* elements;
@@ -828,7 +824,7 @@ midori_web_widget_window_object_cleared_cb (GtkWidget*         web_widget,
     gchar* exception;
     gchar* message;
 
-    uri = webkit_web_frame_get_uri (web_frame);
+    uri = katze_object_get_string (web_widget, "uri");
     if (!uri)
         return;
 
@@ -875,8 +871,8 @@ midori_web_widget_window_object_cleared_cb (GtkWidget*         web_widget,
 
 /**
  * midori_addons_new:
- * @web_widget: a web widget
  * @kind: the kind of addon
+ * @web_widget: a web widget
  *
  * Creates a new addons widget.
  *
@@ -886,11 +882,11 @@ midori_web_widget_window_object_cleared_cb (GtkWidget*         web_widget,
  * Return value: a new #MidoriAddons
  **/
 GtkWidget*
-midori_addons_new (GtkWidget*      web_widget,
-                   MidoriAddonKind kind)
+midori_addons_new (MidoriAddonKind kind,
+                   GtkWidget*      web_widget)
 {
     MidoriAddons* addons;
-    #if HAVE_GIO
+    #if GLIB_CHECK_VERSION (2, 16, 0)
     GSList* directories;
     GSList* list;
     GFile* directory;
@@ -901,13 +897,17 @@ midori_addons_new (GtkWidget*      web_widget,
     g_return_val_if_fail (GTK_IS_WIDGET (web_widget), NULL);
 
     addons = g_object_new (MIDORI_TYPE_ADDONS,
-                           /* "web-widget", web_widget,
-                           "kind", kind, */ NULL);
-    addons->web_widget = web_widget;
-    midori_addons_set_kind (addons, kind);
+                           "kind", kind,
+                           "web-widget", web_widget,
+                           NULL);
+
+    if (kind == MIDORI_ADDON_USER_SCRIPTS || kind == MIDORI_ADDON_USER_STYLES)
+        g_signal_connect (addons->web_widget, "context-ready",
+            G_CALLBACK (midori_web_widget_context_ready_cb), addons);
+
     midori_addons_update_elements (addons);
 
-    #if HAVE_GIO
+    #if GLIB_CHECK_VERSION (2, 16, 0)
     directories = _addons_get_directories (addons);
     list = directories;
     while (directories)
@@ -933,85 +933,6 @@ midori_addons_new (GtkWidget*      web_widget,
     return GTK_WIDGET (addons);
 }
 
-/**
- * midori_addons_set_web_widget:
- * @addons: a #MidoriAddons
- * @web_widget: a web widget
- *
- * Sets the assigned web widget. Basically any widget
- * with a window-object-cleared qualifies as such.
- *
- * Note: This may only be set once.
- **/
-void
-midori_addons_set_web_widget (MidoriAddons* addons,
-                              GtkWidget*    web_widget)
-{
-    g_return_if_fail (MIDORI_IS_ADDONS (addons));
-    g_return_if_fail (GTK_IS_WIDGET (addons));
-    g_return_if_fail (g_signal_lookup ("window-object-cleared", G_TYPE_FROM_INSTANCE (web_widget)));
-
-    /* FIXME: Implement this */
-}
-
-/**
- * midori_addons_get_web_widget:
- * @addons: a #MidoriAddons
- *
- * Determines the assigned web widget.
- *
- * Return value: a web widget
- **/
-GtkWidget*
-midori_addons_get_web_widget (MidoriAddons* addons)
-{
-    return addons->web_widget;
-}
-
-/**
- * midori_addons_set_kind:
- * @addons: a #MidoriAddons
- * @kind: a #MidoriAddonKind
- *
- * Sets the kind of addons.
- *
- * Note: This may only be set once.
- **/
-void
-midori_addons_set_kind (MidoriAddons*   addons,
-                        MidoriAddonKind kind)
-{
-    g_return_if_fail (MIDORI_IS_ADDONS (addons));
-    g_return_if_fail (addons->kind == MIDORI_ADDON_NONE);
-
-    if (kind == MIDORI_ADDON_NONE)
-        return;
-
-    g_return_if_fail (addons->web_widget);
-
-    addons->kind = kind;
-
-    if (kind == MIDORI_ADDON_USER_SCRIPTS || kind == MIDORI_ADDON_USER_STYLES)
-        g_signal_connect (addons->web_widget, "window-object-cleared",
-            G_CALLBACK (midori_web_widget_window_object_cleared_cb), addons);
-
-    g_object_notify (G_OBJECT (addons), "kind");
-}
-
-/**
- * midori_addons_get_kind:
- * @addons: a #MidoriAddons
- *
- * Determines the kind of addons.
- *
- * Return value: a #MidoriAddonKind
- **/
-MidoriAddonKind
-midori_addons_get_kind (MidoriAddons* addons)
-{
-    return addons->kind;
-}
-
 /**
  * midori_addons_get_toolbar:
  * @addons: a #MidoriAddons
@@ -1039,8 +960,8 @@ midori_addons_get_toolbar (MidoriAddons* addons)
         gtk_widget_show (GTK_WIDGET (toolitem));
 
         /* enable button */
-        toolitem = gtk_tool_button_new_from_stock (STOCK_ENABLE);
-        gtk_tool_item_set_is_important (toolitem, TRUE);
+        toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_YES);
+        gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem), _("_Enable"));
         g_signal_connect (toolitem, "clicked",
             G_CALLBACK (midori_addons_button_status_clicked_cb), addons);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
@@ -1048,8 +969,8 @@ midori_addons_get_toolbar (MidoriAddons* addons)
         gtk_widget_show (GTK_WIDGET (toolitem));
 
         /* disable button */
-        toolitem = gtk_tool_button_new_from_stock (STOCK_DISABLE);
-        gtk_tool_item_set_is_important (toolitem, TRUE);
+        toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_NO);
+        gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem), _("_Disable"));
         g_signal_connect (toolitem, "clicked",
             G_CALLBACK (midori_addons_button_status_clicked_cb), addons);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
@@ -1110,6 +1031,7 @@ midori_addons_update_elements (MidoriAddons* addons)
     GSList* list;
     struct AddonElement* element;
 
+    g_return_if_fail (MIDORI_IS_ADDONS (addons));
     g_return_if_fail (addons->kind != MIDORI_ADDON_NONE);
 
     /* FIXME: would GHashTable be better? */
