@@ -36,13 +36,24 @@ struct _MidoriApp
     KatzeArray* trash;
     KatzeArray* search_engines;
     KatzeArray* history;
+    KatzeArray* extensions;
 
     gpointer instance;
 };
 
-G_DEFINE_TYPE (MidoriApp, midori_app, G_TYPE_OBJECT)
+struct _MidoriAppClass
+{
+    GObjectClass parent_class;
 
-static MidoriApp* _midori_app_singleton = NULL;
+    /* Signals */
+    void
+    (*add_browser)            (MidoriApp*     app,
+                               MidoriBrowser* browser);
+    void
+    (*quit)                   (MidoriApp*     app);
+};
+
+G_DEFINE_TYPE (MidoriApp, midori_app, G_TYPE_OBJECT)
 
 enum
 {
@@ -52,8 +63,9 @@ enum
     PROP_BOOKMARKS,
     PROP_TRASH,
     PROP_SEARCH_ENGINES,
-    PROP_BROWSER,
     PROP_HISTORY,
+    PROP_EXTENSIONS,
+    PROP_BROWSER,
     PROP_BROWSER_COUNT
 };
 
@@ -66,11 +78,6 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
-static GObject*
-midori_app_constructor (GType                  type,
-                        guint                  n_construct_properties,
-                        GObjectConstructParam* construct_properties);
-
 static void
 midori_app_finalize (GObject* object);
 
@@ -86,323 +93,6 @@ midori_app_get_property (GObject*    object,
                          GValue*     value,
                          GParamSpec* pspec);
 
-static void
-midori_app_class_init (MidoriAppClass* class)
-{
-    GObjectClass* gobject_class;
-
-    signals[ADD_BROWSER] = g_signal_new (
-        "add-browser",
-        G_TYPE_FROM_CLASS (class),
-        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
-        G_STRUCT_OFFSET (MidoriAppClass, add_browser),
-        0,
-        NULL,
-        g_cclosure_marshal_VOID__OBJECT,
-        G_TYPE_NONE, 1,
-        MIDORI_TYPE_BROWSER);
-
-    signals[QUIT] = g_signal_new (
-        "quit",
-        G_TYPE_FROM_CLASS (class),
-        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
-        G_STRUCT_OFFSET (MidoriAppClass, quit),
-        0,
-        NULL,
-        g_cclosure_marshal_VOID__VOID,
-        G_TYPE_NONE, 0);
-
-    gobject_class = G_OBJECT_CLASS (class);
-    gobject_class->constructor = midori_app_constructor;
-    gobject_class->finalize = midori_app_finalize;
-    gobject_class->set_property = midori_app_set_property;
-    gobject_class->get_property = midori_app_get_property;
-
-    class->add_browser = midori_app_add_browser;
-    class->quit = midori_app_quit;
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_SETTINGS,
-                                     g_param_spec_object (
-                                     "settings",
-                                     "Settings",
-                                     "The associated settings",
-                                     MIDORI_TYPE_WEB_SETTINGS,
-                                     G_PARAM_READWRITE));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_BOOKMARKS,
-                                     g_param_spec_object (
-                                     "bookmarks",
-                                     "Bookmarks",
-                                     "The bookmarks folder, containing all bookmarks",
-                                     KATZE_TYPE_ARRAY,
-                                     G_PARAM_READWRITE));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_TRASH,
-                                     g_param_spec_object (
-                                     "trash",
-                                     "Trash",
-                                     "The trash, collecting recently closed tabs and windows",
-                                     KATZE_TYPE_ARRAY,
-                                     G_PARAM_READWRITE));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_SEARCH_ENGINES,
-                                     g_param_spec_object (
-                                     "search-engines",
-                                     "Search Engines",
-                                     "The list of search engines",
-                                     KATZE_TYPE_ARRAY,
-                                     G_PARAM_READWRITE));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_BROWSER,
-                                     g_param_spec_object (
-                                     "browser",
-                                     "Browser",
-                                     "The current browser",
-                                     MIDORI_TYPE_BROWSER,
-                                     G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_BROWSER_COUNT,
-                                     g_param_spec_uint (
-                                     "browser-count",
-                                     "Browser Count",
-                                     "The current number of browsers",
-                                     0, G_MAXUINT, 0,
-                                     G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_HISTORY,
-                                     g_param_spec_object (
-                                     "history",
-                                     "History",
-                                     "The list of history items",
-                                     KATZE_TYPE_ARRAY,
-                                     G_PARAM_READWRITE));
-
-
-}
-
-static GObject*
-midori_app_constructor (GType                  type,
-                        guint                  n_construct_properties,
-                        GObjectConstructParam* construct_properties)
-{
-    if (_midori_app_singleton)
-        return g_object_ref (_midori_app_singleton);
-    else
-        return G_OBJECT_CLASS (midori_app_parent_class)->constructor (
-            type, n_construct_properties, construct_properties);
-}
-
-#if HAVE_UNIQUE
-static UniqueResponse
-midori_browser_message_received_cb (UniqueApp*         instance,
-                                    UniqueCommand      command,
-                                    UniqueMessageData* message,
-                                    guint              time,
-                                    MidoriApp*         app)
-{
-  UniqueResponse response;
-  MidoriBrowser* browser;
-  gchar** uris;
-
-  switch (command)
-  {
-  case UNIQUE_ACTIVATE:
-      gtk_window_set_screen (GTK_WINDOW (app->browser),
-                             unique_message_data_get_screen (message));
-      gtk_window_present (GTK_WINDOW (app->browser));
-      response = UNIQUE_RESPONSE_OK;
-      break;
-  case UNIQUE_NEW:
-      browser = g_object_new (MIDORI_TYPE_BROWSER,
-                              "settings", app->settings,
-                              "bookmarks", app->bookmarks,
-                              "trash", app->trash,
-                              "search-engines", app->search_engines,
-                              "history", app->history,
-                              NULL);
-      /* FIXME: Should open the homepage according to settings */
-      midori_browser_add_uri (browser, "");
-      midori_app_add_browser (app, browser);
-      gtk_window_set_screen (GTK_WINDOW (app->browser),
-                             unique_message_data_get_screen (message));
-      gtk_widget_show (GTK_WIDGET (browser));
-      response = UNIQUE_RESPONSE_OK;
-      break;
-  case UNIQUE_OPEN:
-      gtk_window_set_screen (GTK_WINDOW (app->browser),
-                             unique_message_data_get_screen (message));
-      gtk_window_present (GTK_WINDOW (app->browser));
-      uris = unique_message_data_get_uris (message);
-      if (!uris)
-          response = UNIQUE_RESPONSE_FAIL;
-      else
-      {
-          while (*uris)
-          {
-              midori_browser_set_current_page (app->browser,
-                  midori_browser_add_uri (app->browser, *uris));
-              uris++;
-          }
-          /* g_strfreev (uris); */
-          response = UNIQUE_RESPONSE_OK;
-      }
-      break;
-  default:
-      response = UNIQUE_RESPONSE_FAIL;
-      break;
-  }
-
-  return response;
-}
-#endif
-
-static void
-midori_app_init (MidoriApp* app)
-{
-    #if HAVE_UNIQUE
-    gchar* display_name;
-    gchar* instance_name;
-    guint i, n;
-    #endif
-
-    g_assert (!_midori_app_singleton);
-
-    _midori_app_singleton = app;
-
-    app->accel_group = gtk_accel_group_new ();
-
-    app->settings = midori_web_settings_new ();
-    app->bookmarks = NULL;
-    app->trash = NULL;
-    app->search_engines = NULL;
-    app->history = NULL;
-
-    #if HAVE_UNIQUE
-    display_name = g_strdup (gdk_display_get_name (gdk_display_get_default ()));
-    n = strlen (display_name);
-    for (i = 0; i < n; i++)
-        if (display_name[i] == ':' || display_name[i] == '.')
-            display_name[i] = '_';
-    instance_name = g_strdup_printf ("de.twotoasts.midori_%s", display_name);
-    app->instance = unique_app_new (instance_name, NULL);
-    g_free (instance_name);
-    g_free (display_name);
-    g_signal_connect (app->instance, "message-received",
-                      G_CALLBACK (midori_browser_message_received_cb), app);
-    #else
-    app->instance = NULL;
-    #endif
-}
-
-static void
-midori_app_finalize (GObject* object)
-{
-    MidoriApp* app = MIDORI_APP (object);
-
-    g_list_free (app->browsers);
-    g_object_unref (app->accel_group);
-
-    if (app->settings)
-        g_object_unref (app->settings);
-    if (app->bookmarks)
-        g_object_unref (app->bookmarks);
-    if (app->trash)
-        g_object_unref (app->trash);
-    if (app->search_engines)
-        g_object_unref (app->search_engines);
-    if (app->history)
-        g_object_unref (app->history);
-
-    if (app->instance)
-        g_object_unref (app->instance);
-
-    G_OBJECT_CLASS (midori_app_parent_class)->finalize (object);
-}
-
-static void
-midori_app_set_property (GObject*      object,
-                         guint         prop_id,
-                         const GValue* value,
-                         GParamSpec*   pspec)
-{
-    MidoriApp* app = MIDORI_APP (object);
-
-    switch (prop_id)
-    {
-    case PROP_SETTINGS:
-        katze_object_assign (app->settings, g_value_get_object (value));
-        g_object_ref (app->settings);
-        /* FIXME: Propagate settings to all browsers */
-        break;
-    case PROP_BOOKMARKS:
-        katze_object_assign (app->bookmarks, g_value_get_object (value));
-        g_object_ref (app->bookmarks);
-        /* FIXME: Propagate bookmarks to all browsers */
-        break;
-    case PROP_TRASH:
-        katze_object_assign (app->trash, g_value_get_object (value));
-        g_object_ref (app->trash);
-        /* FIXME: Propagate trash to all browsers */
-        break;
-    case PROP_SEARCH_ENGINES:
-        katze_object_assign (app->search_engines, g_value_get_object (value));
-        g_object_ref (app->search_engines);
-        /* FIXME: Propagate search engines to all browsers */
-        break;
-    case PROP_HISTORY:
-        katze_object_assign (app->history, g_value_get_object (value));
-        /* FIXME: Propagate history to all browsers */
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-        break;
-    }
-}
-
-static void
-midori_app_get_property (GObject*    object,
-                         guint       prop_id,
-                         GValue*     value,
-                         GParamSpec* pspec)
-{
-    MidoriApp* app = MIDORI_APP (object);
-
-    switch (prop_id)
-    {
-    case PROP_SETTINGS:
-        g_value_set_object (value, app->settings);
-        break;
-    case PROP_BOOKMARKS:
-        g_value_set_object (value, app->bookmarks);
-        break;
-    case PROP_TRASH:
-        g_value_set_object (value, app->trash);
-        break;
-    case PROP_SEARCH_ENGINES:
-        g_value_set_object (value, app->search_engines);
-        break;
-    case PROP_HISTORY:
-        g_value_set_object (value, app->history);
-        break;
-    case PROP_BROWSER:
-        g_value_set_object (value, app->browser);
-        break;
-    case PROP_BROWSER_COUNT:
-        g_value_set_uint (value, g_list_length (app->browsers));
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-        break;
-    }
-}
-
 static gboolean
 midori_browser_focus_in_event_cb (MidoriBrowser* browser,
                                   GdkEventFocus* event,
@@ -457,6 +147,363 @@ midori_browser_quit_cb (MidoriBrowser* browser,
     midori_app_quit (app);
 }
 
+void
+_midori_app_add_browser (MidoriApp*     app,
+                         MidoriBrowser* browser)
+{
+    g_return_if_fail (MIDORI_IS_APP (app));
+    g_return_if_fail (MIDORI_IS_BROWSER (browser));
+
+    gtk_window_add_accel_group (GTK_WINDOW (browser), app->accel_group);
+    g_object_connect (browser,
+        "signal::focus-in-event", midori_browser_focus_in_event_cb, app,
+        "signal::new-window", midori_browser_new_window_cb, app,
+        "signal::delete-event", midori_browser_delete_event_cb, app,
+        "signal::destroy", midori_browser_destroy_cb, app,
+        "signal::quit", midori_browser_quit_cb, app,
+        NULL);
+
+    app->browsers = g_list_prepend (app->browsers, browser);
+
+    #if HAVE_UNIQUE
+    if (app->instance)
+        unique_app_watch_window (app->instance, GTK_WINDOW (browser));
+    #endif
+}
+
+static void
+midori_app_class_init (MidoriAppClass* class)
+{
+    GObjectClass* gobject_class;
+
+    signals[ADD_BROWSER] = g_signal_new (
+        "add-browser",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        G_STRUCT_OFFSET (MidoriAppClass, add_browser),
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__OBJECT,
+        G_TYPE_NONE, 1,
+        MIDORI_TYPE_BROWSER);
+
+    signals[QUIT] = g_signal_new (
+        "quit",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        G_STRUCT_OFFSET (MidoriAppClass, quit),
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__VOID,
+        G_TYPE_NONE, 0);
+
+    gobject_class = G_OBJECT_CLASS (class);
+    gobject_class->finalize = midori_app_finalize;
+    gobject_class->set_property = midori_app_set_property;
+    gobject_class->get_property = midori_app_get_property;
+
+    class->add_browser = _midori_app_add_browser;
+    class->quit = midori_app_quit;
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_SETTINGS,
+                                     g_param_spec_object (
+                                     "settings",
+                                     "Settings",
+                                     "The associated settings",
+                                     MIDORI_TYPE_WEB_SETTINGS,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_BOOKMARKS,
+                                     g_param_spec_object (
+                                     "bookmarks",
+                                     "Bookmarks",
+                                     "The bookmarks folder, containing all bookmarks",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_TRASH,
+                                     g_param_spec_object (
+                                     "trash",
+                                     "Trash",
+                                     "The trash, collecting recently closed tabs and windows",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_SEARCH_ENGINES,
+                                     g_param_spec_object (
+                                     "search-engines",
+                                     "Search Engines",
+                                     "The list of search engines",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_HISTORY,
+                                     g_param_spec_object (
+                                     "history",
+                                     "History",
+                                     "The list of history items",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_EXTENSIONS,
+                                     g_param_spec_object (
+                                     "extensions",
+                                     "Extensions",
+                                     "The list of extensions",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_BROWSER,
+                                     g_param_spec_object (
+                                     "browser",
+                                     "Browser",
+                                     "The current browser",
+                                     MIDORI_TYPE_BROWSER,
+                                     G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_BROWSER_COUNT,
+                                     g_param_spec_uint (
+                                     "browser-count",
+                                     "Browser Count",
+                                     "The current number of browsers",
+                                     0, G_MAXUINT, 0,
+                                     G_PARAM_READABLE));
+}
+
+#if HAVE_UNIQUE
+static UniqueResponse
+midori_browser_message_received_cb (UniqueApp*         instance,
+                                    UniqueCommand      command,
+                                    UniqueMessageData* message,
+                                    guint              time,
+                                    MidoriApp*         app)
+{
+  UniqueResponse response;
+  MidoriBrowser* browser;
+  gchar** uris;
+  MidoriNewPage open_external_pages_in;
+  gboolean first;
+
+  switch (command)
+  {
+  case UNIQUE_ACTIVATE:
+      gtk_window_set_screen (GTK_WINDOW (app->browser),
+                             unique_message_data_get_screen (message));
+      gtk_window_present (GTK_WINDOW (app->browser));
+      response = UNIQUE_RESPONSE_OK;
+      break;
+  case UNIQUE_NEW:
+      browser = g_object_new (MIDORI_TYPE_BROWSER,
+                              "settings", app->settings,
+                              "bookmarks", app->bookmarks,
+                              "trash", app->trash,
+                              "search-engines", app->search_engines,
+                              "history", app->history,
+                              NULL);
+      /* FIXME: Should open the homepage according to settings */
+      midori_browser_add_uri (browser, "");
+      midori_browser_activate_action (browser, "Location");
+      midori_app_add_browser (app, browser);
+      gtk_window_set_screen (GTK_WINDOW (app->browser),
+                             unique_message_data_get_screen (message));
+      gtk_widget_show (GTK_WIDGET (browser));
+      response = UNIQUE_RESPONSE_OK;
+      break;
+  case UNIQUE_OPEN:
+      gtk_window_set_screen (GTK_WINDOW (app->browser),
+                             unique_message_data_get_screen (message));
+      gtk_window_present (GTK_WINDOW (app->browser));
+      uris = unique_message_data_get_uris (message);
+      if (!uris)
+          response = UNIQUE_RESPONSE_FAIL;
+      else
+      {
+          g_object_get (app->settings, "open-external-pages-in",
+              &open_external_pages_in, NULL);
+          if (open_external_pages_in == MIDORI_NEW_PAGE_WINDOW)
+          {
+              browser = g_object_new (MIDORI_TYPE_BROWSER,
+                  "settings", app->settings,
+                  "bookmarks", app->bookmarks,
+                  "trash", app->trash,
+                  "search-engines", app->search_engines,
+                  "history", app->history,
+                  NULL);
+              midori_app_add_browser (app, browser);
+              gtk_window_set_screen (GTK_WINDOW (app->browser),
+                                     unique_message_data_get_screen (message));
+              gtk_widget_show (GTK_WIDGET (browser));
+          }
+          else
+              browser = app->browser;
+          first = (open_external_pages_in == MIDORI_NEW_PAGE_CURRENT);
+          while (*uris)
+          {
+              if (first)
+              {
+                  midori_browser_set_current_uri (browser, *uris);
+                  first = FALSE;
+              }
+              else
+                  midori_browser_set_current_page (browser,
+                      midori_browser_add_uri (browser, *uris));
+              uris++;
+          }
+          /* g_strfreev (uris); */
+          response = UNIQUE_RESPONSE_OK;
+      }
+      break;
+  default:
+      response = UNIQUE_RESPONSE_FAIL;
+      break;
+  }
+
+  return response;
+}
+#endif
+
+static void
+midori_app_init (MidoriApp* app)
+{
+    #if HAVE_UNIQUE
+    gchar* display_name;
+    gchar* instance_name;
+    guint i, n;
+    #endif
+
+    app->accel_group = gtk_accel_group_new ();
+
+    app->settings = midori_web_settings_new ();
+    app->bookmarks = NULL;
+    app->trash = NULL;
+    app->search_engines = NULL;
+    app->history = NULL;
+    app->extensions = NULL;
+
+    #if HAVE_UNIQUE
+    display_name = g_strdup (gdk_display_get_name (gdk_display_get_default ()));
+    n = strlen (display_name);
+    for (i = 0; i < n; i++)
+        if (display_name[i] == ':' || display_name[i] == '.')
+            display_name[i] = '_';
+    instance_name = g_strdup_printf ("de.twotoasts.midori_%s", display_name);
+    app->instance = unique_app_new (instance_name, NULL);
+    g_free (instance_name);
+    g_free (display_name);
+    g_signal_connect (app->instance, "message-received",
+                      G_CALLBACK (midori_browser_message_received_cb), app);
+    #else
+    app->instance = NULL;
+    #endif
+}
+
+static void
+midori_app_finalize (GObject* object)
+{
+    MidoriApp* app = MIDORI_APP (object);
+
+    g_list_free (app->browsers);
+    g_object_unref (app->accel_group);
+
+    katze_object_assign (app->settings, NULL);
+    katze_object_assign (app->bookmarks, NULL);
+    katze_object_assign (app->trash, NULL);
+    katze_object_assign (app->search_engines, NULL);
+    katze_object_assign (app->history, NULL);
+    katze_object_assign (app->extensions, NULL);
+
+    katze_object_assign (app->instance, NULL);
+
+    G_OBJECT_CLASS (midori_app_parent_class)->finalize (object);
+}
+
+static void
+midori_app_set_property (GObject*      object,
+                         guint         prop_id,
+                         const GValue* value,
+                         GParamSpec*   pspec)
+{
+    MidoriApp* app = MIDORI_APP (object);
+
+    switch (prop_id)
+    {
+    case PROP_SETTINGS:
+        katze_object_assign (app->settings, g_value_dup_object (value));
+        /* FIXME: Propagate settings to all browsers */
+        break;
+    case PROP_BOOKMARKS:
+        katze_object_assign (app->bookmarks, g_value_dup_object (value));
+        /* FIXME: Propagate bookmarks to all browsers */
+        break;
+    case PROP_TRASH:
+        katze_object_assign (app->trash, g_value_dup_object (value));
+        /* FIXME: Propagate trash to all browsers */
+        break;
+    case PROP_SEARCH_ENGINES:
+        katze_object_assign (app->search_engines, g_value_dup_object (value));
+        /* FIXME: Propagate search engines to all browsers */
+        break;
+    case PROP_HISTORY:
+        katze_object_assign (app->history, g_value_dup_object (value));
+        /* FIXME: Propagate history to all browsers */
+        break;
+    case PROP_EXTENSIONS:
+        katze_object_assign (app->extensions, g_value_dup_object (value));
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
+static void
+midori_app_get_property (GObject*    object,
+                         guint       prop_id,
+                         GValue*     value,
+                         GParamSpec* pspec)
+{
+    MidoriApp* app = MIDORI_APP (object);
+
+    switch (prop_id)
+    {
+    case PROP_SETTINGS:
+        g_value_set_object (value, app->settings);
+        break;
+    case PROP_BOOKMARKS:
+        g_value_set_object (value, app->bookmarks);
+        break;
+    case PROP_TRASH:
+        g_value_set_object (value, app->trash);
+        break;
+    case PROP_SEARCH_ENGINES:
+        g_value_set_object (value, app->search_engines);
+        break;
+    case PROP_HISTORY:
+        g_value_set_object (value, app->history);
+        break;
+    case PROP_EXTENSIONS:
+        g_value_set_object (value, app->extensions);
+        break;
+    case PROP_BROWSER:
+        g_value_set_object (value, app->browser);
+        break;
+    case PROP_BROWSER_COUNT:
+        g_value_set_uint (value, g_list_length (app->browsers));
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
 /**
  * midori_app_new:
  *
@@ -611,70 +658,7 @@ midori_app_add_browser (MidoriApp*     app,
     g_return_if_fail (MIDORI_IS_APP (app));
     g_return_if_fail (MIDORI_IS_BROWSER (browser));
 
-    gtk_window_add_accel_group (GTK_WINDOW (browser), app->accel_group);
-    g_object_connect (browser,
-        "signal::focus-in-event", midori_browser_focus_in_event_cb, app,
-        "signal::new-window", midori_browser_new_window_cb, app,
-        "signal::delete-event", midori_browser_delete_event_cb, app,
-        "signal::destroy", midori_browser_destroy_cb, app,
-        "signal::quit", midori_browser_quit_cb, app,
-        NULL);
-
-    app->browsers = g_list_prepend (app->browsers, browser);
-
-    #if HAVE_UNIQUE
-    if (app->instance)
-        unique_app_watch_window (app->instance, GTK_WINDOW (browser));
-    #endif
-}
-
-/**
- * midori_app_get_settings:
- * @app: a #MidoriApp
- *
- * Retrieves the #MidoriWebSettings of the app.
- *
- * Return value: the assigned #MidoriWebSettings
- **/
-MidoriWebSettings*
-midori_app_get_settings (MidoriApp* app)
-{
-    g_return_val_if_fail (MIDORI_IS_APP (app), NULL);
-
-    return app->settings;
-}
-
-/**
- * midori_app_set_settings:
- * @app: a #MidoriApp
- *
- * Assigns the #MidoriWebSettings to the app.
- *
- * Return value: the assigned #MidoriWebSettings
- **/
-void
-midori_app_set_settings (MidoriApp*         app,
-                         MidoriWebSettings* settings)
-{
-    g_return_if_fail (MIDORI_IS_APP (app));
-
-    g_object_set (app, "settings", settings, NULL);
-}
-
-/**
- * midori_app_get_trash:
- * @app: a #MidoriApp
- *
- * Retrieves the trash of the app.
- *
- * Return value: the assigned #MidoriTrash
- **/
-KatzeArray*
-midori_app_get_trash (MidoriApp* app)
-{
-    g_return_val_if_fail (MIDORI_IS_APP (app), NULL);
-
-    return app->trash;
+    g_signal_emit (app, signals[ADD_BROWSER], 0, browser);
 }
 
 /**
