@@ -35,18 +35,6 @@ G_BEGIN_DECLS
 typedef struct _MidoriApp                MidoriApp;
 typedef struct _MidoriAppClass           MidoriAppClass;
 
-struct _MidoriAppClass
-{
-    GObjectClass parent_class;
-
-    /* Signals */
-    void
-    (*add_browser)            (MidoriApp*     app,
-                               MidoriBrowser* browser);
-    void
-    (*quit)                   (MidoriApp*     app);
-};
-
 GType
 midori_app_get_type               (void);
 
@@ -70,16 +58,6 @@ void
 midori_app_add_browser            (MidoriApp*         app,
                                    MidoriBrowser*     browser);
 
-MidoriWebSettings*
-midori_app_get_settings           (MidoriApp*         app);
-
-void
-midori_app_set_settings           (MidoriApp*         app,
-                                   MidoriWebSettings* settings);
-
-KatzeArray*
-midori_app_get_trash              (MidoriApp*         app);
-
 void
 midori_app_quit                   (MidoriApp*         app);
 
