@@ -30,6 +30,15 @@ struct _MidoriPanel
     GtkMenu*   menu;
 };
 
+struct _MidoriPanelClass
+{
+    GtkHBoxClass parent_class;
+
+    /* Signals */
+    gboolean
+    (*close)                  (MidoriPanel*   panel);
+};
+
 G_DEFINE_TYPE (MidoriPanel, midori_panel, GTK_TYPE_HBOX)
 
 enum
@@ -66,7 +75,7 @@ midori_panel_get_property (GObject*    object,
                            GParamSpec* pspec);
 
 static gboolean
-midori_panel_close_cb (MidoriPanel* panel)
+midori_panel_close (MidoriPanel* panel)
 {
     gtk_widget_hide (GTK_WIDGET (panel));
     return FALSE;
@@ -109,6 +118,9 @@ midori_cclosure_marshal_BOOLEAN__VOID (GClosure*     closure,
 static void
 midori_panel_class_init (MidoriPanelClass* class)
 {
+    GObjectClass* gobject_class;
+    GParamFlags flags;
+
 
     signals[CLOSE] = g_signal_new (
         "close",
@@ -124,21 +136,21 @@ midori_panel_class_init (MidoriPanelClass* class)
         "switch-page",
         G_TYPE_FROM_CLASS (class),
         (GSignalFlags)(G_SIGNAL_RUN_LAST),
-        G_STRUCT_OFFSET (MidoriPanelClass, switch_page),
+        0,
         0,
         NULL,
         g_cclosure_marshal_VOID__INT,
         G_TYPE_NONE, 1,
         G_TYPE_INT);
 
-    class->close = midori_panel_close_cb;
-
-    GObjectClass* gobject_class = G_OBJECT_CLASS (class);
+    gobject_class = G_OBJECT_CLASS (class);
     gobject_class->finalize = midori_panel_finalize;
     gobject_class->set_property = midori_panel_set_property;
     gobject_class->get_property = midori_panel_get_property;
 
-    GParamFlags flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
+    class->close = midori_panel_close;
+
+    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
 
     g_object_class_install_property (gobject_class,
                                      PROP_SHADOW_TYPE,
@@ -180,6 +192,10 @@ midori_panel_button_close_clicked_cb (GtkWidget*   toolitem,
 static void
 midori_panel_init (MidoriPanel* panel)
 {
+    GtkWidget* vbox;
+    GtkWidget* labelbar;
+    GtkToolItem* toolitem;
+
     /* Create the sidebar */
     panel->toolbar = gtk_toolbar_new ();
     gtk_toolbar_set_style (GTK_TOOLBAR (panel->toolbar), GTK_TOOLBAR_BOTH);
@@ -189,14 +205,14 @@ midori_panel_init (MidoriPanel* panel)
                                  GTK_ORIENTATION_VERTICAL);
     gtk_box_pack_start (GTK_BOX (panel), panel->toolbar, FALSE, FALSE, 0);
     gtk_widget_show_all (panel->toolbar);
-    GtkWidget* vbox = gtk_vbox_new (FALSE, 0);
+    vbox = gtk_vbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (panel), vbox, TRUE, TRUE, 0);
 
     /* Create the titlebar */
-    GtkWidget* labelbar = gtk_toolbar_new ();
+    labelbar = gtk_toolbar_new ();
     gtk_toolbar_set_icon_size (GTK_TOOLBAR (labelbar), GTK_ICON_SIZE_MENU);
     gtk_toolbar_set_style (GTK_TOOLBAR (labelbar), GTK_TOOLBAR_ICONS);
-    GtkToolItem* toolitem = gtk_tool_item_new ();
+    toolitem = gtk_tool_item_new ();
     gtk_tool_item_set_expand (toolitem, TRUE);
     panel->toolbar_label = gtk_label_new (NULL);
     gtk_misc_set_alignment (GTK_MISC (panel->toolbar_label), 0, 0.5);
@@ -208,7 +224,11 @@ midori_panel_init (MidoriPanel* panel)
     gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (toolitem), _("Close panel"));
     g_signal_connect (toolitem, "clicked",
         G_CALLBACK (midori_panel_button_close_clicked_cb), panel);
+    #if HAVE_OSX
+    gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, 0);
+    #else
     gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, -1);
+    #endif
     gtk_box_pack_start (GTK_BOX (vbox), labelbar, FALSE, FALSE, 0);
     gtk_widget_show_all (vbox);
 
@@ -257,7 +277,7 @@ midori_panel_set_property (GObject*      object,
                                    g_value_get_enum (value));
         break;
     case PROP_MENU:
-        katze_object_assign (panel->menu, g_value_get_object (value));
+        katze_object_assign (panel->menu, g_value_dup_object (value));
         /* FIXME: Move existing items to the new menu */
         break;
     case PROP_PAGE:
@@ -311,12 +331,28 @@ midori_panel_new (void)
     return GTK_WIDGET (panel);
 }
 
+/**
+ * midori_panel_set_compact:
+ * @compact: %TRUE if the panel should be compact
+ *
+ * Determines if the panel should be compact.
+ **/
+void
+midori_panel_set_compact (MidoriPanel* panel,
+                          gboolean     compact)
+{
+    g_return_if_fail (MIDORI_IS_PANEL (panel));
+
+    gtk_toolbar_set_style (GTK_TOOLBAR (panel->toolbar),
+        compact ? GTK_TOOLBAR_ICONS : GTK_TOOLBAR_BOTH);
+}
+
 static void
 midori_panel_menu_item_activate_cb (GtkWidget*   widget,
                                     MidoriPanel* panel)
 {
     GtkWidget* child;
-    GtkToolItem* toolitem;
+    GtkToggleToolButton* toolitem;
     guint n;
 
     child = g_object_get_data (G_OBJECT (widget), "page");
@@ -324,10 +360,10 @@ midori_panel_menu_item_activate_cb (GtkWidget*   widget,
 
     if (toolitem)
     {
-        /* Unset the button before setting it ensures that
+        /* Unsetting the button before setting it ensures that
            it will emit signals even if it was active before */
-        gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (toolitem), FALSE);
-        gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (toolitem), TRUE);
+        gtk_toggle_tool_button_set_active (toolitem, FALSE);
+        gtk_toggle_tool_button_set_active (toolitem, TRUE);
     }
     else
     {
@@ -341,41 +377,38 @@ midori_panel_menu_item_activate_cb (GtkWidget*   widget,
 /**
  * midori_panel_append_page:
  * @panel: a #MidoriPanel
- * @child: the child widget
+ * @viewable: a viewable widget
  * @toolbar: a toolbar widget, or %NULL
  * @stock_id: a stock ID
  * @label: a string to use as the label
  *
  * Appends a new page to the panel. If @toolbar is specified it will
- * be packed above @child.
+ * be packed above @viewable.
  *
  * In the case of an error, -1 is returned.
  *
  * Return value: the index of the new page, or -1
  **/
 gint
-midori_panel_append_page (MidoriPanel* panel,
-                          GtkWidget*   child,
-                          GtkWidget*   toolbar,
-                          const gchar* stock_id,
-                          const gchar* label)
+midori_panel_append_page (MidoriPanel*    panel,
+                          MidoriViewable* viewable)
 {
     GtkWidget* scrolled;
-    GtkWidget* widget;
     GObjectClass* gobject_class;
-    guint n;
+    GtkWidget* widget;
+    GtkWidget* toolbar;
+    const gchar* label;
+    const gchar* stock_id;
     GtkToolItem* toolitem;
     GtkWidget* image;
     GtkWidget* menuitem;
+    guint n;
 
     g_return_val_if_fail (MIDORI_IS_PANEL (panel), -1);
-    g_return_val_if_fail (GTK_IS_WIDGET (child), -1);
-    g_return_val_if_fail (!toolbar || GTK_IS_WIDGET (toolbar), -1);
-    g_return_val_if_fail (stock_id != NULL, -1);
-    g_return_val_if_fail (label != NULL, -1);
+    g_return_val_if_fail (MIDORI_IS_VIEWABLE (viewable), -1);
 
-    if (GTK_IS_SCROLLED_WINDOW (child))
-        scrolled = child;
+    if (GTK_IS_SCROLLED_WINDOW (viewable))
+        scrolled = (GtkWidget*)viewable;
     else
     {
         scrolled = gtk_scrolled_window_new (NULL, NULL);
@@ -384,35 +417,38 @@ midori_panel_append_page (MidoriPanel* panel,
                                         GTK_POLICY_AUTOMATIC);
         GTK_WIDGET_SET_FLAGS (scrolled, GTK_CAN_FOCUS);
         gtk_widget_show (scrolled);
-        gobject_class = G_OBJECT_GET_CLASS (child);
+        gobject_class = G_OBJECT_GET_CLASS (viewable);
         if (GTK_WIDGET_CLASS (gobject_class)->set_scroll_adjustments_signal)
-            widget = child;
+            widget = (GtkWidget*)viewable;
         else
         {
             widget = gtk_viewport_new (NULL, NULL);
             gtk_widget_show (widget);
-            gtk_container_add (GTK_CONTAINER (widget), child);
+            gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (viewable));
         }
         gtk_container_add (GTK_CONTAINER (scrolled), widget);
     }
     gtk_container_add (GTK_CONTAINER (panel->notebook), scrolled);
 
-    if (!toolbar)
-        toolbar = gtk_event_box_new ();
+    toolbar = midori_viewable_get_toolbar (viewable);
     gtk_widget_show (toolbar);
     gtk_container_add (GTK_CONTAINER (panel->toolbook), toolbar);
 
     n = midori_panel_page_num (panel, scrolled);
+    label = midori_viewable_get_label (viewable);
+    stock_id = midori_viewable_get_stock_id (viewable);
 
-    g_object_set_data (G_OBJECT (child), "label", (gchar*)label);
-
-    toolitem = gtk_radio_tool_button_new (panel->group);
+    toolitem = gtk_radio_tool_button_new_from_stock (panel->group, stock_id);
     panel->group = gtk_radio_tool_button_get_group (GTK_RADIO_TOOL_BUTTON (
-                                                   toolitem));
+                                                    toolitem));
     image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
     gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toolitem), image);
-    gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem), label);
-    g_object_set_data (G_OBJECT (toolitem), "page", child);
+    if (label)
+    {
+        gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem), label);
+        gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem), label);
+    }
+    g_object_set_data (G_OBJECT (toolitem), "page", viewable);
     g_signal_connect (toolitem, "clicked",
                       G_CALLBACK (midori_panel_menu_item_activate_cb), panel);
     gtk_widget_show_all (GTK_WIDGET (toolitem));
@@ -422,7 +458,7 @@ midori_panel_append_page (MidoriPanel* panel,
     {
         menuitem = gtk_image_menu_item_new_from_stock (stock_id, NULL);
         gtk_widget_show (menuitem);
-        g_object_set_data (G_OBJECT (menuitem), "page", child);
+        g_object_set_data (G_OBJECT (menuitem), "page", viewable);
         g_object_set_data (G_OBJECT (menuitem), "toolitem", toolitem);
         g_signal_connect (menuitem, "activate",
                           G_CALLBACK (midori_panel_menu_item_activate_cb),
@@ -538,9 +574,12 @@ gint
 midori_panel_page_num (MidoriPanel* panel,
                        GtkWidget*   child)
 {
-    g_return_val_if_fail (MIDORI_IS_PANEL (panel), -1);
+    GtkWidget* scrolled;
 
-    GtkWidget* scrolled = _midori_panel_scrolled_for_child (panel, child);
+    g_return_val_if_fail (MIDORI_IS_PANEL (panel), -1);
+    g_return_val_if_fail (GTK_IS_WIDGET (child), -1);
+
+    scrolled = _midori_panel_scrolled_for_child (panel, child);
     return gtk_notebook_page_num (GTK_NOTEBOOK (panel->notebook), scrolled);
 }
 
@@ -558,14 +597,156 @@ void
 midori_panel_set_current_page (MidoriPanel* panel,
                                gint         n)
 {
+    GtkWidget* viewable;
+
     g_return_if_fail (MIDORI_IS_PANEL (panel));
 
     gtk_notebook_set_current_page (GTK_NOTEBOOK (panel->toolbook), n);
     gtk_notebook_set_current_page (GTK_NOTEBOOK (panel->notebook), n);
-    GtkWidget* child = midori_panel_get_nth_page (panel, n);
-    if (child)
+
+    if ((viewable = midori_panel_get_nth_page (panel, n)))
     {
-        const gchar* label = g_object_get_data (G_OBJECT (child), "label");
+        const gchar* label;
+
+        label = midori_viewable_get_label (MIDORI_VIEWABLE (viewable));
         g_object_set (panel->toolbar_label, "label", label, NULL);
     }
 }
+
+typedef struct
+{
+    GtkAlignment parent_instance;
+
+    gchar* label;
+    gchar* stock_id;
+    GtkWidget* toolbar;
+} MidoriDummyViewable;
+
+typedef struct
+{
+    GtkAlignmentClass parent_class;
+}  MidoriDummyViewableClass;
+
+#define MIDORI_TYPE_DUMMY_VIEWABLE (midori_dummy_viewable_get_type ())
+#define MIDORI_DUMMY_VIEWABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
+    MIDORI_TYPE_DUMMY_VIEWABLE, MidoriDummyViewable))
+#define MIDORI_IS_DUMMY_VIEWABLE(obj) \
+    (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_DUMMY_VIEWABLE))
+
+static void
+midori_dummy_viewable_iface_init (MidoriViewableIface* iface);
+
+static void
+midori_dummy_viewable_finalize (GObject* object);
+
+G_DEFINE_TYPE_WITH_CODE (MidoriDummyViewable, midori_dummy_viewable,
+                         GTK_TYPE_ALIGNMENT,
+                         G_IMPLEMENT_INTERFACE (MIDORI_TYPE_VIEWABLE,
+                                                midori_dummy_viewable_iface_init));
+
+static void
+midori_dummy_viewable_class_init (MidoriDummyViewableClass* class)
+{
+    GObjectClass* gobject_class;
+
+    gobject_class = G_OBJECT_CLASS (class);
+    gobject_class->finalize = midori_dummy_viewable_finalize;
+}
+
+static const gchar*
+midori_dummy_viewable_get_label (MidoriViewable* viewable)
+{
+    return MIDORI_DUMMY_VIEWABLE (viewable)->label;
+}
+
+static const gchar*
+midori_dummy_viewable_get_stock_id (MidoriViewable* viewable)
+{
+    return MIDORI_DUMMY_VIEWABLE (viewable)->stock_id;
+}
+
+static GtkWidget*
+midori_dummy_viewable_get_toolbar (MidoriViewable* viewable)
+{
+    return MIDORI_DUMMY_VIEWABLE (viewable)->toolbar;
+}
+
+static void
+midori_dummy_viewable_iface_init (MidoriViewableIface* iface)
+{
+    iface->get_stock_id = midori_dummy_viewable_get_stock_id;
+    iface->get_label = midori_dummy_viewable_get_label;
+    iface->get_toolbar = midori_dummy_viewable_get_toolbar;
+}
+
+static void
+midori_dummy_viewable_init (MidoriDummyViewable* viewable)
+{
+    viewable->stock_id = NULL;
+    viewable->label = NULL;
+    viewable->toolbar = NULL;
+}
+
+static void
+midori_dummy_viewable_finalize (GObject* object)
+{
+    MidoriDummyViewable* viewable = MIDORI_DUMMY_VIEWABLE (object);
+
+    katze_assign (viewable->stock_id, NULL);
+    katze_assign (viewable->label, NULL);
+
+    G_OBJECT_CLASS (midori_dummy_viewable_parent_class)->finalize (object);
+}
+
+static GtkWidget*
+midori_dummy_viewable_new (const gchar* stock_id,
+                       const gchar* label,
+                       GtkWidget*   toolbar)
+{
+    GtkWidget* viewable = g_object_new (MIDORI_TYPE_DUMMY_VIEWABLE, NULL);
+
+    MIDORI_DUMMY_VIEWABLE (viewable)->stock_id = g_strdup (stock_id);
+    MIDORI_DUMMY_VIEWABLE (viewable)->label = g_strdup (label);
+    MIDORI_DUMMY_VIEWABLE (viewable)->toolbar = toolbar;
+
+    return viewable;
+}
+
+/**
+ * midori_panel_append_widget:
+ * @panel: a #MidoriPanel
+ * @widget: the child widget
+ * @stock_id: a stock ID
+ * @label: a string to use as the label, or %NULL
+ * @toolbar: a toolbar widget, or %NULL
+ *
+ * Appends an arbitrary widget to the panel by wrapping it
+ * in a #MidoriViewable created on the fly.
+ *
+ * Actually implementing #MidoriViewable instead of using
+ * this convenience is recommended.
+ *
+ * In the case of an error, -1 is returned.
+ *
+ * Return value: the index of the new page, or -1
+ **/
+gint
+midori_panel_append_widget (MidoriPanel* panel,
+                            GtkWidget*   widget,
+                            const gchar* stock_id,
+                            const gchar* label,
+                            GtkWidget*   toolbar)
+{
+    GtkWidget* viewable;
+
+    g_return_val_if_fail (MIDORI_IS_PANEL (panel), -1);
+    g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
+
+    g_return_val_if_fail (stock_id != NULL, -1);
+    g_return_val_if_fail (!toolbar || GTK_IS_WIDGET (toolbar), -1);
+
+    viewable = midori_dummy_viewable_new (stock_id, label, toolbar);
+    gtk_widget_show (viewable);
+    gtk_container_add (GTK_CONTAINER (viewable), widget);
+    return midori_panel_append_page (panel, MIDORI_VIEWABLE (viewable));
+}
