@@ -11,8 +11,13 @@
 
 #include "midori-preferences.h"
 
+#if HAVE_CONFIG_H
+    #include <config.h>
+#endif
+
 #include "sokoke.h"
 
+#include <string.h>
 #include <glib/gi18n.h>
 
 struct _MidoriPreferences
@@ -72,10 +77,6 @@ midori_preferences_response_cb (MidoriPreferences* preferences,
         gtk_widget_destroy (GTK_WIDGET (preferences));
 }
 
-#ifdef GDK_WINDOWING_QUARTZ
-    #define USE_OSX_STYLE
-#endif
-
 static void
 midori_preferences_init (MidoriPreferences* preferences)
 {
@@ -83,14 +84,14 @@ midori_preferences_init (MidoriPreferences* preferences)
 
     preferences->notebook = NULL;
 
-    dialog_title = g_strdup_printf (_("%s Preferences"),
+    dialog_title = g_strdup_printf (_("Preferences for %s"),
                                     g_get_application_name ());
     g_object_set (preferences,
                   "icon-name", GTK_STOCK_PREFERENCES,
                   "title", dialog_title,
                   "has-separator", FALSE,
                   NULL);
-    #ifndef USE_OSX_STYLE
+    #ifndef HAVE_OSX
     gtk_dialog_add_buttons (GTK_DIALOG (preferences),
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -99,7 +100,7 @@ midori_preferences_init (MidoriPreferences* preferences)
     g_signal_connect (preferences, "response",
                       G_CALLBACK (midori_preferences_response_cb), NULL);
 
-    /* TODO: Do we want tooltips for explainations or can we omit that? */
+    /* TODO: Do we want tooltips for explanations or can we omit that? */
     g_free (dialog_title);
 }
 
@@ -166,13 +167,15 @@ GtkWidget*
 midori_preferences_new (GtkWindow*         parent,
                         MidoriWebSettings* settings)
 {
+    MidoriPreferences* preferences;
+
     g_return_val_if_fail (GTK_IS_WINDOW (parent), NULL);
     g_return_val_if_fail (MIDORI_IS_WEB_SETTINGS (settings), NULL);
 
-    MidoriPreferences* preferences = g_object_new (MIDORI_TYPE_PREFERENCES,
-                                                   "transient-for", parent,
-                                                   "settings", settings,
-                                                   NULL);
+    preferences = g_object_new (MIDORI_TYPE_PREFERENCES,
+                                "transient-for", parent,
+                                "settings", settings,
+                                NULL);
 
     return GTK_WIDGET (preferences);
 }
@@ -182,30 +185,38 @@ proxy_download_manager_icon_cb (GtkWidget*     entry,
                                 GdkEventFocus* event,
                                 GtkImage*      icon)
 {
-    const gchar* program;
+    const gchar* command;
+    gchar* first_space;
+    gchar* first_part;
     gchar* path;
 
-    program = gtk_entry_get_text (GTK_ENTRY (entry));
-    path = g_find_program_in_path (program);
+    command = gtk_entry_get_text (GTK_ENTRY (entry));
+    if ((first_space = strstr (command, " ")))
+        first_part = g_strndup (command, first_space - command);
+    else
+        first_part = g_strdup (command);
+    path = g_find_program_in_path (first_part);
 
     if (path)
     {
         if (gtk_icon_theme_has_icon (gtk_icon_theme_get_for_screen (
-            gtk_widget_get_screen (entry)), program))
-            gtk_image_set_from_icon_name (icon, program, GTK_ICON_SIZE_MENU);
+            gtk_widget_get_screen (entry)), first_part))
+            gtk_image_set_from_icon_name (icon, first_part, GTK_ICON_SIZE_MENU);
         else
             gtk_image_set_from_stock (icon, GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU);
         g_free (path);
     }
-    else if (program && *program)
+    else if (first_part && *first_part)
         gtk_image_set_from_stock (icon, GTK_STOCK_STOP, GTK_ICON_SIZE_MENU);
     else
         gtk_image_clear (icon);
 
+    g_free (first_part);
+
     return FALSE;
 }
 
-#ifdef USE_OSX_STYLE
+#ifdef HAVE_OSX
 static void
 midori_preferences_help_clicked_cb (GtkWidget* button,
                                     GtkDialog* dialog)
@@ -230,7 +241,7 @@ midori_preferences_add_toolbutton (GtkWidget*   toolbar,
                                    const gchar* label,
                                    GtkWidget*   page)
 {
-#ifdef USE_OSX_STYLE
+#ifdef HAVE_OSX
     *toolbutton = GTK_WIDGET (*toolbutton ? gtk_radio_tool_button_new_from_widget (
         GTK_RADIO_TOOL_BUTTON (*toolbutton)) : gtk_radio_tool_button_new (NULL));
     gtk_tool_button_set_label (GTK_TOOL_BUTTON (*toolbutton), label);
@@ -274,7 +285,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     preferences->notebook = gtk_notebook_new ();
     gtk_container_set_border_width (GTK_CONTAINER (preferences->notebook), 6);
 
-    #ifdef USE_OSX_STYLE
+    #ifdef HAVE_OSX
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (preferences->notebook), FALSE);
     gtk_notebook_set_show_border (GTK_NOTEBOOK (preferences->notebook), FALSE);
     toolbar = gtk_toolbar_new ();
@@ -282,8 +293,9 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
                         toolbar, FALSE, FALSE, 0);
-    #endif
+    #else
     toolbar = NULL;
+    #endif
     toolbutton = NULL;
 
     sizegroup = NULL;
@@ -332,11 +344,11 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     FILLED_ADD (button, 1, 2, 0, 1);
     label = katze_property_label (settings, "homepage");
     INDENTED_ADD (label, 0, 1, 1, 2);
+    /* TODO: We need something like "use current website" */
     entry = katze_property_proxy (settings, "homepage", NULL);
     FILLED_ADD (entry, 1, 2, 1, 2);
-    /* TODO: We need something like "use current website" */
     FRAME_NEW (_("Transfers"));
-    TABLE_NEW (1, 2);
+    TABLE_NEW (3, 2);
     label = katze_property_label (settings, "download-folder");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "download-folder", "folder");
@@ -355,12 +367,26 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     g_signal_connect (entry, "focus-out-event",
         G_CALLBACK (proxy_download_manager_icon_cb), button);
     FILLED_ADD (hbox, 1, 2, 1, 2);
+    label = katze_property_label (settings, "text-editor");
+    INDENTED_ADD (label, 0, 1, 2, 3);
+    hbox = gtk_hbox_new (FALSE, 4);
+    button = gtk_image_new ();
+    gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
+        GTK_ICON_SIZE_MENU, &icon_width, &icon_height);
+    gtk_widget_set_size_request (button, icon_width, icon_height);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
+    entry = katze_property_proxy (settings, "text-editor", NULL);
+    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+    proxy_download_manager_icon_cb (entry, NULL, GTK_IMAGE (button));
+    g_signal_connect (entry, "focus-out-event",
+        G_CALLBACK (proxy_download_manager_icon_cb), button);
+    FILLED_ADD (hbox, 1, 2, 2, 3);
 
     /* Page "Appearance" */
     PAGE_NEW (GTK_STOCK_SELECT_FONT, _("Appearance"));
     FRAME_NEW (_("Font settings"));
     TABLE_NEW (5, 2);
-    label = katze_property_label (settings, "default-font-family");
+    label = gtk_label_new (_("Default Font Family"));
     INDENTED_ADD (label, 0, 1, 0, 1);
     hbox = gtk_hbox_new (FALSE, 4);
     button = katze_property_proxy (settings, "default-font-family", "font");
@@ -368,7 +394,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     entry = katze_property_proxy (settings, "default-font-size", NULL);
     gtk_box_pack_end (GTK_BOX (hbox), entry, FALSE, FALSE, 4);
     FILLED_ADD (hbox, 1, 2, 0, 1);
-    label = katze_property_label (settings, "minimum-font-size");
+    label = gtk_label_new (_("Minimum Font Size"));
     INDENTED_ADD (label, 0, 1, 1, 2);
     entry = katze_property_proxy (settings, "minimum-font-size", NULL);
     INDENTED_ADD (entry, 1, 2, 1, 2);
@@ -382,26 +408,39 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     FRAME_NEW (_("Features"));
     TABLE_NEW (6, 2);
     button = katze_property_proxy (settings, "auto-load-images", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Load images automatically"));
     INDENTED_ADD (button, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "auto-shrink-images", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Shrink images automatically"));
     SPANNED_ADD (button, 1, 2, 0, 1);
     button = katze_property_proxy (settings, "print-backgrounds", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Print background images"));
     INDENTED_ADD (button, 0, 1, 1, 2);
     button = katze_property_proxy (settings, "resizable-text-areas", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Resizable text areas"));
     SPANNED_ADD (button, 1, 2, 1, 2);
     button = katze_property_proxy (settings, "enable-scripts", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Enable scripts"));
     INDENTED_ADD (button, 0, 1, 2, 3);
     button = katze_property_proxy (settings, "enable-plugins", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Enable plugins"));
     SPANNED_ADD (button, 1, 2, 2, 3);
+    #ifdef WEBKIT_CHECK_VERSION
+    #if WEBKIT_CHECK_VERSION (1, 0, 3)
+    button = katze_property_proxy (settings, "enable-developer-extras", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Enable developer tools"));
+    SPANNED_ADD (button, 0, 1, 3, 4);
+    #endif
+    #endif
     label = katze_property_label (settings, "location-entry-search");
-    INDENTED_ADD (label, 0, 1, 3, 4);
+    INDENTED_ADD (label, 0, 1, 4, 5);
     entry = katze_property_proxy (settings, "location-entry-search", NULL);
-    FILLED_ADD (entry, 1, 2, 3, 4);
+    FILLED_ADD (entry, 1, 2, 4, 5);
 
     /* Page "Interface" */
     PAGE_NEW (GTK_STOCK_CONVERT, _("Interface"));
     FRAME_NEW (_("Navigationbar"));
-    TABLE_NEW (4, 2);
+    TABLE_NEW (3, 2);
     INDENTED_ADD (katze_property_label (settings, "toolbar-style"), 0, 1, 0, 1);
     button = katze_property_proxy (settings, "toolbar-style", NULL);
     FILLED_ADD (button, 1, 2, 0, 1);
@@ -415,27 +454,34 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         "ReloadStop, ZoomIn, ZoomOut, Back, Forward, Homepage, Search, Trash"
         "</span>");
     FILLED_ADD (button, 0, 2, 2, 3);
-    button = katze_property_proxy (settings, "always-show-tabbar", NULL);
-    INDENTED_ADD (button, 0, 1, 3, 4);
     FRAME_NEW (_("Browsing"));
-    TABLE_NEW (3, 2);
+    TABLE_NEW (4, 2);
+    label = katze_property_label (settings, "open-external-pages-in");
+    INDENTED_ADD (label, 0, 1, 0, 1);
+    button = katze_property_proxy (settings, "open-external-pages-in", NULL);
+    FILLED_ADD (button, 1, 2, 0, 1);
     /* label = katze_property_label (settings, "open-new-pages-in");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "open-new-pages-in", NULL);
     FILLED_ADD (button, 1, 2, 0, 1); */
-    button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
+    button = katze_property_proxy (settings, "always-show-tabbar", NULL);
     INDENTED_ADD (button, 0, 1, 1, 2);
+    button = katze_property_proxy (settings, "compact-sidepanel", NULL);
+    INDENTED_ADD (button, 1, 2, 1, 2);
+    button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
+    INDENTED_ADD (button, 0, 1, 2, 3);
     button = katze_property_proxy (settings, "open-tabs-in-the-background", NULL);
-    WIDGET_ADD (button, 1, 2, 1, 2);
+    WIDGET_ADD (button, 1, 2, 2, 3);
     /* button = katze_property_proxy (settings, "open-popups-in-tabs", NULL);
     SPANNED_ADD (button, 0, 1, 2, 3);*/
     button = katze_property_proxy (settings, "open-tabs-next-to-current", NULL);
-    WIDGET_ADD (button, 0, 1, 2, 3);
+    WIDGET_ADD (button, 0, 1, 3, 4);
     button = katze_property_proxy (settings, "close-buttons-on-tabs", NULL);
-    WIDGET_ADD (button, 1, 2, 2, 3);
+    WIDGET_ADD (button, 1, 2, 3, 4);
 
     /* Page "Network" */
-    /*PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
+    #if 0
+    PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
     FRAME_NEW (_("Network"));
     TABLE_NEW (2, 2);
     label = katze_property_label (settings, "http-proxy");
@@ -449,11 +495,13 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("MB")),
                         FALSE, FALSE, 0);
-    FILLED_ADD (hbox, 1, 2, 1, 2);*/
+    FILLED_ADD (hbox, 1, 2, 1, 2);
+    #endif
 
     /* Page "Privacy" */
     PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
-    /*FRAME_NEW (_("Web Cookies"));
+    #if 0
+    FRAME_NEW (_("Web Cookies"));
     TABLE_NEW (3, 2);
     label = katze_property_label (settings, "accept-cookies");
     INDENTED_ADD (label, 0, 1, 0, 1);
@@ -468,7 +516,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("days")),
                         FALSE, FALSE, 0);
-    FILLED_ADD (hbox, 1, 2, 2, 3);*/
+    FILLED_ADD (hbox, 1, 2, 2, 3);
+    #endif
     FRAME_NEW (_("History"));
     TABLE_NEW (3, 2);
     button = katze_property_proxy (settings, "remember-last-visited-pages", NULL);
@@ -487,7 +536,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     g_object_unref (sizegroup);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
                         preferences->notebook, FALSE, FALSE, 4);
-    #ifdef USE_OSX_STYLE
+    #ifdef HAVE_OSX
     GtkWidget* icon;
     hbox = gtk_hbox_new (FALSE, 0);
     button = gtk_button_new ();
