@@ -25,6 +25,8 @@ struct _MidoriSearchAction
     KatzeItem* current_item;
     gchar* text;
 
+    KatzeNet* net;
+
     GtkWidget* last_proxy;
 
     GtkWidget* dialog;
@@ -205,6 +207,8 @@ midori_search_action_init (MidoriSearchAction* search_action)
     search_action->current_item = NULL;
     search_action->text = NULL;
 
+    search_action->net = katze_net_new ();
+
     search_action->last_proxy = NULL;
 
     search_action->dialog = NULL;
@@ -218,7 +222,9 @@ midori_search_action_finalize (GObject* object)
 {
     MidoriSearchAction* search_action = MIDORI_SEARCH_ACTION (object);
 
-    g_free (search_action->text);
+    katze_assign (search_action->text, NULL);
+
+    katze_object_assign (search_action->net, NULL);
 
     G_OBJECT_CLASS (midori_search_action_parent_class)->finalize (object);
 }
@@ -419,8 +425,8 @@ midori_search_action_icon_released_cb (GtkWidget*           entry,
     guint n, i;
     GtkWidget* menuitem;
     KatzeItem* item;
-    GdkPixbuf* pixbuf;
-    GtkWidget* icon;
+    GdkPixbuf* icon;
+    GtkWidget* image;
 
     search_engines = MIDORI_SEARCH_ACTION (action)->search_engines;
     menu = gtk_menu_new ();
@@ -432,11 +438,12 @@ midori_search_action_icon_released_cb (GtkWidget*           entry,
             item = katze_array_get_nth_item (search_engines, i);
             menuitem = gtk_image_menu_item_new_with_label (
                 katze_item_get_name (item));
-            pixbuf = sokoke_web_icon (katze_item_get_icon (item),
-                                      GTK_ICON_SIZE_MENU, menuitem);
-            icon = gtk_image_new_from_pixbuf (pixbuf);
-            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
-            g_object_unref (pixbuf);
+            image = gtk_image_new ();
+            icon = katze_net_load_icon (MIDORI_SEARCH_ACTION (action)->net,
+                katze_item_get_uri (item), NULL, entry, NULL);
+            gtk_image_set_from_pixbuf (GTK_IMAGE (image), icon);
+            g_object_unref (icon);
+            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
             g_object_set_data (G_OBJECT (menuitem), "engine", item);
             g_signal_connect (menuitem, "activate",
@@ -456,8 +463,8 @@ midori_search_action_icon_released_cb (GtkWidget*           entry,
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
     gtk_widget_show (menuitem);
     menuitem = gtk_image_menu_item_new_with_mnemonic (_("_Manage Search Engines"));
-    icon = gtk_image_new_from_stock (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
-    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
+    image = gtk_image_new_from_stock (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
+    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
     g_signal_connect (menuitem, "activate",
         G_CALLBACK (midori_search_action_manage_activate_cb), action);
@@ -478,6 +485,32 @@ midori_search_action_scroll_event_cb (GtkWidget*          entry,
     return FALSE;
 }
 
+static void
+midori_search_action_set_entry_icon (MidoriSearchAction* search_action,
+                                     GtkWidget*          entry)
+{
+    GdkPixbuf* icon;
+
+    if (search_action->current_item)
+    {
+        icon = katze_net_load_icon (search_action->net,
+            katze_item_get_uri (search_action->current_item),
+            NULL, entry, NULL);
+        gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
+                                             GTK_ICON_ENTRY_PRIMARY, icon);
+        g_object_unref (icon);
+        sokoke_entry_set_default_text (GTK_ENTRY (entry),
+            katze_item_get_name (search_action->current_item));
+    }
+    else
+    {
+        gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
+                                            GTK_ICON_ENTRY_PRIMARY,
+                                            GTK_STOCK_FIND);
+        sokoke_entry_set_default_text (GTK_ENTRY (entry), "");
+    }
+}
+
 static void
 midori_search_action_connect_proxy (GtkAction* action,
                                       GtkWidget* proxy)
@@ -493,6 +526,8 @@ midori_search_action_connect_proxy (GtkAction* action,
         alignment = gtk_bin_get_child (GTK_BIN (proxy));
         entry = gtk_bin_get_child (GTK_BIN (alignment));
 
+        midori_search_action_set_entry_icon (MIDORI_SEARCH_ACTION (action),
+                                             entry);
         g_object_connect (entry,
                           "signal::key-press-event",
                           midori_search_action_key_press_event_cb, action,
@@ -656,7 +691,6 @@ midori_search_action_set_current_item (MidoriSearchAction* search_action,
     GSList* proxies;
     GtkWidget* alignment;
     GtkWidget* entry;
-    GdkPixbuf* pixbuf;
 
     g_return_if_fail (MIDORI_IS_SEARCH_ACTION (search_action));
     g_return_if_fail (!item || KATZE_IS_ITEM (item));
@@ -677,16 +711,7 @@ midori_search_action_set_current_item (MidoriSearchAction* search_action,
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
         entry = gtk_bin_get_child (GTK_BIN (alignment));
 
-        pixbuf = sokoke_web_icon (item ? katze_item_get_icon (item) : NULL,
-                                  GTK_ICON_SIZE_MENU, entry);
-        gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
-                                             GTK_ICON_ENTRY_PRIMARY, pixbuf);
-        g_object_unref (pixbuf);
-        if (item)
-            sokoke_entry_set_default_text (GTK_ENTRY (entry),
-                                           katze_item_get_name (item));
-        else
-            sokoke_entry_set_default_text (GTK_ENTRY (entry), "");
+        midori_search_action_set_entry_icon (search_action, entry);
     }
     while ((proxies = g_slist_next (proxies)));
 }
@@ -698,22 +723,19 @@ midori_search_action_dialog_render_icon_cb (GtkTreeViewColumn* column,
                                             GtkTreeIter*       iter,
                                             GtkWidget*         treeview)
 {
+    KatzeNet* net;
     KatzeItem* item;
-    const gchar* icon;
-    GdkPixbuf* pixbuf;
+    GdkPixbuf* icon;
 
     gtk_tree_model_get (model, iter, 0, &item, -1);
 
-    icon = katze_item_get_icon (item);
-    if (icon)
-    {
-        pixbuf = sokoke_web_icon (icon, GTK_ICON_SIZE_DND, treeview);
-        g_object_set (renderer, "pixbuf", pixbuf, NULL);
-        if (pixbuf)
-            g_object_unref (pixbuf);
-    }
-    else
-        g_object_set (renderer, "pixbuf", NULL, NULL);
+    /* FIXME: Use the net of the MidoriSearchAction */
+    net = katze_net_new ();
+    icon = katze_net_load_icon (net, katze_item_get_uri (item),
+        NULL, treeview, NULL);
+    g_object_set (renderer, "pixbuf", icon, NULL);
+    g_object_unref (icon);
+    g_object_unref (net);
 }
 
 static void
@@ -1040,8 +1062,10 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     dialog = gtk_dialog_new_with_buttons (dialog_title,
         toplevel ? GTK_WINDOW (toplevel) : NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+        #ifndef HAVE_OSX
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+        #endif
         NULL);
     g_signal_connect (dialog, "destroy",
                       G_CALLBACK (gtk_widget_destroyed), &search_action->dialog);
@@ -1126,6 +1150,22 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     button = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
     gtk_widget_set_sensitive (button, FALSE);
     gtk_box_pack_end (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+
+    #ifdef HAVE_OSX
+    GtkWidget* icon;
+    hbox = gtk_hbox_new (FALSE, 0);
+    button = gtk_button_new ();
+    icon = gtk_image_new_from_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image (GTK_BUTTON (button), icon);
+    /* TODO: Implement some kind of help function */
+    gtk_widget_set_sensitive (button, FALSE);
+    /* g_signal_connect (button, "clicked",
+        G_CALLBACK (midori_search_action_dialog_help_clicked_cb), dialog); */
+    gtk_box_pack_end (GTK_BOX (hbox),
+        button, FALSE, FALSE, 4);
+    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+        hbox, FALSE, FALSE, 0);
+    #endif
     gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
 
     g_object_connect (search_action->search_engines,
