@@ -51,7 +51,7 @@ GType
 midori_view_get_type                   (void);
 
 GtkWidget*
-midori_view_new                        (void);
+midori_view_new                        (KatzeNet*          net);
 
 void
 midori_view_set_settings               (MidoriView*        view,
@@ -171,6 +171,11 @@ void
 midori_view_set_highlight_text_matches (MidoriView*        view,
                                         gboolean           highlight);
 
+gboolean
+midori_view_execute_script             (MidoriView*        view,
+                                        const gchar*       script,
+                                        gchar**            exception);
+
 G_END_DECLS
 
 #endif /* __MIDORI_VIEW_H__ */
