@@ -26,7 +26,6 @@ struct _MidoriWebSettings
     gint last_panel_position;
     gint last_panel_page;
     gint last_web_search;
-    gchar* last_pageholder_uri;
 
     gboolean show_menubar;
     gboolean show_navigationbar;
@@ -36,21 +35,20 @@ struct _MidoriWebSettings
 
     MidoriToolbarStyle toolbar_style;
     gchar* toolbar_items;
-    gboolean always_show_tabbar;
-    gboolean show_new_tab;
-    gboolean show_homepage;
-    gboolean show_web_search;
-    gboolean show_trash;
+    gboolean compact_sidepanel;
 
     MidoriStartup load_on_startup;
     gchar* homepage;
     gchar* download_folder;
     gchar* download_manager;
+    gchar* text_editor;
     gchar* location_entry_search;
     MidoriPreferredEncoding preferred_encoding;
 
+    gboolean always_show_tabbar;
     gboolean close_buttons_on_tabs;
     MidoriNewPage open_new_pages_in;
+    MidoriNewPage open_external_pages_in;
     gboolean middle_click_opens_selection;
     gboolean open_tabs_in_the_background;
     gboolean open_tabs_next_to_current;
@@ -81,7 +79,6 @@ enum
     PROP_LAST_PANEL_POSITION,
     PROP_LAST_PANEL_PAGE,
     PROP_LAST_WEB_SEARCH,
-    PROP_LAST_PAGEHOLDER_URI,
 
     PROP_SHOW_MENUBAR,
     PROP_SHOW_NAVIGATIONBAR,
@@ -91,21 +88,20 @@ enum
 
     PROP_TOOLBAR_STYLE,
     PROP_TOOLBAR_ITEMS,
-    PROP_ALWAYS_SHOW_TABBAR,
-    PROP_SHOW_NEW_TAB,
-    PROP_SHOW_HOMEPAGE,
-    PROP_SHOW_WEB_SEARCH,
-    PROP_SHOW_TRASH,
+    PROP_COMPACT_SIDEPANEL,
 
     PROP_LOAD_ON_STARTUP,
     PROP_HOMEPAGE,
     PROP_DOWNLOAD_FOLDER,
     PROP_DOWNLOAD_MANAGER,
+    PROP_TEXT_EDITOR,
     PROP_LOCATION_ENTRY_SEARCH,
     PROP_PREFERRED_ENCODING,
 
+    PROP_ALWAYS_SHOW_TABBAR,
     PROP_CLOSE_BUTTONS_ON_TABS,
     PROP_OPEN_NEW_PAGES_IN,
+    PROP_OPEN_EXTERNAL_PAGES_IN,
     PROP_MIDDLE_CLICK_OPENS_SELECTION,
     PROP_OPEN_TABS_IN_THE_BACKGROUND,
     PROP_OPEN_TABS_NEXT_TO_CURRENT,
@@ -232,12 +228,15 @@ midori_web_settings_get_property (GObject*    object,
 static void
 midori_web_settings_class_init (MidoriWebSettingsClass* class)
 {
-    GObjectClass* gobject_class = G_OBJECT_CLASS (class);
+    GObjectClass* gobject_class;
+    GParamFlags flags;
+
+    gobject_class = G_OBJECT_CLASS (class);
     gobject_class->finalize = midori_web_settings_finalize;
     gobject_class->set_property = midori_web_settings_set_property;
     gobject_class->get_property = midori_web_settings_get_property;
 
-    GParamFlags flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
+    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
 
     g_object_class_install_property (gobject_class,
                                      PROP_REMEMBER_LAST_WINDOW_SIZE,
@@ -279,6 +278,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_LAST_PANEL_PAGE,
                                      g_param_spec_int (
                                      "last-panel-page",
+        /* i18n: The internal index of the last opened panel */
                                      _("Last panel page"),
                                      _("The last saved panel page"),
                                      0, G_MAXINT, 0,
@@ -293,15 +293,6 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      0, G_MAXINT, 0,
                                      flags));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_LAST_PAGEHOLDER_URI,
-                                     g_param_spec_string (
-                                     "last-pageholder-uri",
-                                     _("Last pageholder URI"),
-                                     _("The URI last opened in the pageholder"),
-                                     "",
-                                     flags));
-
 
     g_object_class_install_property (gobject_class,
                                      PROP_SHOW_MENUBAR,
@@ -369,51 +360,14 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      flags));
 
     g_object_class_install_property (gobject_class,
-                                     PROP_ALWAYS_SHOW_TABBAR,
+                                     PROP_COMPACT_SIDEPANEL,
                                      g_param_spec_boolean (
-                                     "always-show-tabbar",
-                                     _("Always Show Tabbar"),
-                                     _("Always show the tabbar"),
+                                     "compact-sidepanel",
+                                     _("Compact Sidepanel"),
+                                     _("Whether to make the sidepanel compact"),
                                      FALSE,
                                      flags));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_SHOW_NEW_TAB,
-                                     g_param_spec_boolean (
-                                     "show-new-tab",
-                                     _("Show New Tab"),
-                                     _("Show the New Tab button in the toolbar"),
-                                     TRUE,
-                                     flags));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_SHOW_HOMEPAGE,
-                                     g_param_spec_boolean (
-                                     "show-homepage",
-                                     _("Show Homepage"),
-                                     _("Show the Homepage button in the toolbar"),
-                                     TRUE,
-                                     flags));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_SHOW_WEB_SEARCH,
-                                     g_param_spec_boolean (
-                                     "show-web-search",
-                                     _("Show Web search"),
-                                     _("Show the Web search entry in the toolbar"),
-                                     TRUE,
-                                     flags));
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_SHOW_TRASH,
-                                     g_param_spec_boolean (
-                                     "show-trash",
-                                     _("Show Trash"),
-                                     _("Show the Trash button in the toolbar"),
-                                     TRUE,
-                                     flags));
-
-
 
     g_object_class_install_property (gobject_class,
                                      PROP_LOAD_ON_STARTUP,
@@ -452,6 +406,15 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      NULL,
                                      flags));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_TEXT_EDITOR,
+                                     g_param_spec_string (
+                                     "text-editor",
+                                     _("Text Editor"),
+                                     _("An external text editor"),
+                                     NULL,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_LOCATION_ENTRY_SEARCH,
                                      g_param_spec_string (
@@ -472,6 +435,15 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      flags));
 
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_ALWAYS_SHOW_TABBAR,
+                                     g_param_spec_boolean (
+                                     "always-show-tabbar",
+                                     _("Always Show Tabbar"),
+                                     _("Always show the tabbar"),
+                                     FALSE,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_CLOSE_BUTTONS_ON_TABS,
                                      g_param_spec_boolean (
@@ -491,6 +463,16 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      MIDORI_NEW_PAGE_TAB,
                                      G_PARAM_READABLE));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_OPEN_EXTERNAL_PAGES_IN,
+                                     g_param_spec_enum (
+                                     "open-external-pages-in",
+                                     _("Open external pages in"),
+                                     _("Where to open externally opened pages"),
+                                     MIDORI_TYPE_NEW_PAGE,
+                                     MIDORI_NEW_PAGE_TAB,
+                                     G_PARAM_READWRITE));
+
     g_object_class_install_property (gobject_class,
                                      PROP_MIDDLE_CLICK_OPENS_SELECTION,
                                      g_param_spec_boolean (
@@ -620,11 +602,14 @@ static void
 notify_default_encoding_cb (GObject*    object,
                             GParamSpec* pspec)
 {
-    MidoriWebSettings* web_settings = MIDORI_WEB_SETTINGS (object);
-
+    MidoriWebSettings* web_settings;
     const gchar* string;
+    const gchar* encoding;
+
+    web_settings = MIDORI_WEB_SETTINGS (object);
+
     g_object_get (object, "default-encoding", &string, NULL);
-    const gchar* encoding = string ? string : "";
+    encoding = string ? string : "";
     if (!strcmp (encoding, "BIG5"))
         web_settings->preferred_encoding = MIDORI_ENCODING_CHINESE;
     else if (!strcmp (encoding, "SHIFT_JIS"))
@@ -683,9 +668,6 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_LAST_WEB_SEARCH:
         web_settings->last_web_search = g_value_get_int (value);
         break;
-    case PROP_LAST_PAGEHOLDER_URI:
-        katze_assign (web_settings->last_pageholder_uri, g_value_dup_string (value));
-        break;
 
     case PROP_SHOW_MENUBAR:
         web_settings->show_menubar = g_value_get_boolean (value);
@@ -709,20 +691,8 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_TOOLBAR_ITEMS:
         katze_assign (web_settings->toolbar_items, g_value_dup_string (value));
         break;
-    case PROP_ALWAYS_SHOW_TABBAR:
-        web_settings->always_show_tabbar = g_value_get_boolean (value);
-        break;
-    case PROP_SHOW_NEW_TAB:
-        web_settings->show_new_tab = g_value_get_boolean (value);
-        break;
-    case PROP_SHOW_HOMEPAGE:
-        web_settings->show_homepage = g_value_get_boolean (value);
-        break;
-    case PROP_SHOW_WEB_SEARCH:
-        web_settings->show_web_search = g_value_get_boolean (value);
-        break;
-    case PROP_SHOW_TRASH:
-        web_settings->show_trash = g_value_get_boolean (value);
+    case PROP_COMPACT_SIDEPANEL:
+        web_settings->compact_sidepanel = g_value_get_boolean (value);
         break;
 
     case PROP_LOAD_ON_STARTUP:
@@ -737,6 +707,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_DOWNLOAD_MANAGER:
         katze_assign (web_settings->download_manager, g_value_dup_string (value));
         break;
+    case PROP_TEXT_EDITOR:
+        katze_assign (web_settings->text_editor, g_value_dup_string (value));
+        break;
     case PROP_LOCATION_ENTRY_SEARCH:
         katze_assign (web_settings->location_entry_search, g_value_dup_string (value));
         break;
@@ -764,12 +737,18 @@ midori_web_settings_set_property (GObject*      object,
         }
         break;
 
+    case PROP_ALWAYS_SHOW_TABBAR:
+        web_settings->always_show_tabbar = g_value_get_boolean (value);
+        break;
     case PROP_CLOSE_BUTTONS_ON_TABS:
         web_settings->close_buttons_on_tabs = g_value_get_boolean (value);
         break;
     case PROP_OPEN_NEW_PAGES_IN:
         web_settings->open_new_pages_in = g_value_get_enum (value);
         break;
+    case PROP_OPEN_EXTERNAL_PAGES_IN:
+        web_settings->open_external_pages_in = g_value_get_enum (value);
+        break;
     case PROP_MIDDLE_CLICK_OPENS_SELECTION:
         web_settings->middle_click_opens_selection = g_value_get_boolean (value);
         break;
@@ -846,9 +825,6 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_LAST_WEB_SEARCH:
         g_value_set_int (value, web_settings->last_web_search);
         break;
-    case PROP_LAST_PAGEHOLDER_URI:
-        g_value_set_string (value, web_settings->last_pageholder_uri);
-        break;
 
     case PROP_SHOW_MENUBAR:
         g_value_set_boolean (value, web_settings->show_menubar);
@@ -872,20 +848,8 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_TOOLBAR_ITEMS:
         g_value_set_string (value, web_settings->toolbar_items);
         break;
-    case PROP_ALWAYS_SHOW_TABBAR:
-        g_value_set_boolean (value, web_settings->always_show_tabbar);
-        break;
-    case PROP_SHOW_NEW_TAB:
-        g_value_set_boolean (value, web_settings->show_new_tab);
-        break;
-    case PROP_SHOW_HOMEPAGE:
-        g_value_set_boolean (value, web_settings->show_homepage);
-        break;
-    case PROP_SHOW_WEB_SEARCH:
-        g_value_set_boolean (value, web_settings->show_web_search);
-        break;
-    case PROP_SHOW_TRASH:
-        g_value_set_boolean (value, web_settings->show_trash);
+    case PROP_COMPACT_SIDEPANEL:
+        g_value_set_boolean (value, web_settings->compact_sidepanel);
         break;
 
     case PROP_LOAD_ON_STARTUP:
@@ -900,6 +864,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_DOWNLOAD_MANAGER:
         g_value_set_string (value, web_settings->download_manager);
         break;
+    case PROP_TEXT_EDITOR:
+        g_value_set_string (value, web_settings->text_editor);
+        break;
     case PROP_LOCATION_ENTRY_SEARCH:
         g_value_set_string (value, web_settings->location_entry_search);
         break;
@@ -907,12 +874,18 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_enum (value, web_settings->preferred_encoding);
         break;
 
+    case PROP_ALWAYS_SHOW_TABBAR:
+        g_value_set_boolean (value, web_settings->always_show_tabbar);
+        break;
     case PROP_CLOSE_BUTTONS_ON_TABS:
         g_value_set_boolean (value, web_settings->close_buttons_on_tabs);
         break;
     case PROP_OPEN_NEW_PAGES_IN:
         g_value_set_enum (value, web_settings->open_new_pages_in);
         break;
+    case PROP_OPEN_EXTERNAL_PAGES_IN:
+        g_value_set_enum (value, web_settings->open_external_pages_in);
+        break;
     case PROP_MIDDLE_CLICK_OPENS_SELECTION:
         g_value_set_boolean (value, web_settings->middle_click_opens_selection);
         break;
