@@ -9,12 +9,12 @@
  See the file COPYING for the full license text.
 */
 
+#include "sokoke.h"
+
 #if HAVE_CONFIG_H
     #include <config.h>
 #endif
 
-#include "sokoke.h"
-
 #if HAVE_UNISTD_H
     #include <unistd.h>
 #endif
@@ -29,7 +29,7 @@ error_dialog (const gchar* short_message,
               const gchar* detailed_message)
 {
     GtkWidget* dialog = gtk_message_dialog_new (
-            NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, short_message);
+        NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", short_message);
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                               "%s", detailed_message);
     gtk_widget_show (dialog);
@@ -48,6 +48,9 @@ sokoke_spawn_program (const gchar* command,
     gchar** argv;
     GError* error;
 
+    g_return_val_if_fail (command != NULL, FALSE);
+    g_return_val_if_fail (argument != NULL, FALSE);
+
     argument_escaped = g_shell_quote (argument);
     if (strstr (command, "%s"))
         command_ready = g_strdup_printf (command, argument_escaped);
@@ -85,14 +88,14 @@ sokoke_magic_uri (const gchar* uri,
 {
     gchar* current_dir;
     gchar* result;
+    gchar** parts;
     gchar* search;
     const gchar* search_uri;
-    gchar** parts;
     KatzeItem* item;
 
     g_return_val_if_fail (uri, NULL);
-    if (search_engines)
-        g_return_val_if_fail (katze_array_is_a (search_engines, KATZE_TYPE_ITEM), NULL);
+    g_return_val_if_fail (!search_engines ||
+        katze_array_is_a (search_engines, KATZE_TYPE_ITEM), NULL);
 
     /* Add file:// if we have a local path */
     if (g_path_is_absolute (uri))
@@ -111,9 +114,23 @@ sokoke_magic_uri (const gchar* uri,
     if (!strstr (uri, "://"))
     {
         /* Do we have a domain, ip address or localhost? */
-        if (strchr (uri, '.') != NULL || strchr (uri, ':')
-            || !strcmp (uri, "localhost"))
+        search = strchr (uri, ':');
+        if (search && search[0] && !g_ascii_isalpha (search[1]))
+            if (!strchr (search, '.'))
+                return g_strconcat ("http://", uri, NULL);
+        if (!strcmp (uri, "localhost"))
             return g_strconcat ("http://", uri, NULL);
+        parts = g_strsplit (uri, ".", 0);
+        if (!search && parts[0] && parts[1])
+        {
+            search = NULL;
+            if (!(parts[1][1] == '\0' && !g_ascii_isalpha (parts[1][0])))
+                if (!strchr (parts[0], ' ') && !strchr (parts[1], ' '))
+                    search = g_strconcat ("http://", uri, NULL);
+            g_free (parts);
+            if (search)
+                return search;
+        }
         /* We don't want to search? So return early. */
         if (!search_engines)
             return g_strdup (uri);
@@ -135,41 +152,17 @@ sokoke_magic_uri (const gchar* uri,
     return g_strdup (uri);
 }
 
-void
-sokoke_entry_setup_completion (GtkEntry* entry)
-{
-    /* TODO: The current behavior works only with the beginning of strings
-             But we want to match "localhost" with "loc" and "hos" */
-    GtkEntryCompletion* completion = gtk_entry_completion_new ();
-    gtk_entry_completion_set_model (completion,
-        GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING)));
-    gtk_entry_completion_set_text_column (completion, 0);
-    gtk_entry_completion_set_minimum_key_length (completion, 3);
-    gtk_entry_set_completion (entry, completion);
-    /* FIXME: Completion doesn't work well, so it's disabled */
-    gtk_entry_completion_set_popup_completion (completion, FALSE);
-}
-
-void
-sokoke_entry_append_completion (GtkEntry* entry, const gchar* text)
-{
-    GtkEntryCompletion* completion = gtk_entry_get_completion (entry);
-    GtkTreeModel* completion_store = gtk_entry_completion_get_model (completion);
-    GtkTreeIter iter;
-    gtk_list_store_insert (GTK_LIST_STORE (completion_store), &iter, 0);
-    gtk_list_store_set (GTK_LIST_STORE (completion_store), &iter, 0, text, -1);
-}
-
 void
 sokoke_combo_box_add_strings (GtkComboBox* combobox,
                               const gchar* label_first, ...)
 {
+    const gchar* label;
+
     /* Add a number of strings to a combobox, terminated with NULL
        This works only for text comboboxes */
     va_list args;
     va_start (args, label_first);
 
-    const gchar* label;
     for (label = label_first; label; label = va_arg (args, const gchar*))
         gtk_combo_box_append_text (combobox, label);
 
@@ -205,30 +198,32 @@ typedef enum
 {
     SOKOKE_DESKTOP_UNTESTED,
     SOKOKE_DESKTOP_XFCE,
+    SOKOKE_DESKTOP_OSX,
     SOKOKE_DESKTOP_UNKNOWN
 } SokokeDesktop;
 
 static SokokeDesktop
 sokoke_get_desktop (void)
 {
-    #ifdef GDK_WINDOWING_X11
+    #ifdef HAVE_OSX
+    return SOKOKE_DESKTOP_OSX;
+    #else
     static SokokeDesktop desktop = SOKOKE_DESKTOP_UNTESTED;
     if (G_UNLIKELY (desktop == SOKOKE_DESKTOP_UNTESTED))
     {
         /* Are we running in Xfce? */
-        gint result; gchar* out; gchar* err;
-        gboolean success = g_spawn_command_line_sync (
-            "xprop -root _DT_SAVE_MODE | grep -q xfce4",
-            &out, &err, &result, NULL);
-        if (success && !result)
+        gint result;
+        gchar *out = NULL;
+        gboolean success = g_spawn_command_line_sync ("xprop -root _DT_SAVE_MODE",
+            &out, NULL, &result, NULL);
+        if (success && ! result && out != NULL && strstr(out, "xfce4") != NULL)
             desktop = SOKOKE_DESKTOP_XFCE;
         else
             desktop = SOKOKE_DESKTOP_UNKNOWN;
+        g_free(out);
     }
 
     return desktop;
-    #else
-    return SOKOKE_DESKTOP_UNKNOWN;
     #endif
 }
 
@@ -240,17 +235,22 @@ sokoke_xfce_header_new (const gchar* icon,
        This returns NULL if the desktop is not Xfce */
     if (sokoke_get_desktop () == SOKOKE_DESKTOP_XFCE)
     {
-        GtkWidget* entry = gtk_entry_new ();
+        GtkWidget* entry;
         gchar* markup;
-        GtkWidget* xfce_heading = gtk_event_box_new ();
+        GtkWidget* xfce_heading;
+        GtkWidget* hbox;
+        GtkWidget* image;
+        GtkWidget* label;
+
+        xfce_heading = gtk_event_box_new ();
+        entry = gtk_entry_new ();
         gtk_widget_modify_bg (xfce_heading, GTK_STATE_NORMAL,
             &entry->style->base[GTK_STATE_NORMAL]);
-        GtkWidget* hbox = gtk_hbox_new (FALSE, 12);
+        hbox = gtk_hbox_new (FALSE, 12);
         gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-        GtkWidget* image = gtk_image_new_from_icon_name (icon,
-                                                         GTK_ICON_SIZE_DIALOG);
+        image = gtk_image_new_from_icon_name (icon, GTK_ICON_SIZE_DIALOG);
         gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-        GtkWidget* label = gtk_label_new (NULL);
+        label = gtk_label_new (NULL);
         gtk_widget_modify_fg (label, GTK_STATE_NORMAL
          , &entry->style->text[GTK_STATE_NORMAL]);
         markup = g_strdup_printf ("<span size='large' weight='bold'>%s</span>",
@@ -259,6 +259,7 @@ sokoke_xfce_header_new (const gchar* icon,
         gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
         gtk_container_add (GTK_CONTAINER (xfce_heading), hbox);
         g_free (markup);
+        gtk_widget_destroy(entry);
         return xfce_heading;
     }
     return NULL;
@@ -272,11 +273,13 @@ sokoke_superuser_warning_new (void)
     #if HAVE_UNISTD_H
     if (G_UNLIKELY (!geteuid ())) /* effective superuser? */
     {
-        GtkWidget* hbox = gtk_event_box_new ();
+        GtkWidget* hbox;
+        GtkWidget* label;
+
+        hbox = gtk_event_box_new ();
         gtk_widget_modify_bg (hbox, GTK_STATE_NORMAL,
                               &hbox->style->bg[GTK_STATE_SELECTED]);
-        GtkWidget* label = gtk_label_new (
-            _("Warning: You are using a superuser account!"));
+        label = gtk_label_new (_("Warning: You are using a superuser account!"));
         gtk_misc_set_padding (GTK_MISC (label), 0, 2);
         gtk_widget_modify_fg (GTK_WIDGET (label), GTK_STATE_NORMAL,
             &GTK_WIDGET (label)->style->fg[GTK_STATE_SELECTED]);
@@ -443,10 +446,13 @@ sokoke_key_file_save_to_file (GKeyFile*    key_file,
                               const gchar* filename,
                               GError**     error)
 {
-    gchar* data = g_key_file_to_data (key_file, NULL, error);
+    gchar* data;
+    FILE* fp;
+
+    data = g_key_file_to_data (key_file, NULL, error);
     if (!data)
         return FALSE;
-    FILE* fp;
+
     if (!(fp = fopen (filename, "w")))
     {
         *error = g_error_new (G_FILE_ERROR, G_FILE_ERROR_ACCES,
@@ -470,72 +476,6 @@ sokoke_widget_get_text_size (GtkWidget*   widget,
     g_object_unref (layout);
 }
 
-GdkPixbuf*
-sokoke_web_icon (const gchar* icon,
-                 GtkIconSize  size,
-                 GtkWidget*   widget)
-{
-    g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-
-    GdkPixbuf* pixbuf = NULL;
-    if (icon && *icon)
-    {
-        /* TODO: We want to allow http as well, maybe also base64? */
-        const gchar* icon_ready = g_str_has_prefix (icon, "file://")
-            ? &icon[7] : icon;
-        GtkStockItem stock_id;
-        if (gtk_stock_lookup (icon, &stock_id))
-            pixbuf = gtk_widget_render_icon (widget, icon_ready, size, NULL);
-        else
-        {
-            gint width, height;
-            if (gtk_widget_has_screen (widget))
-            {
-                GdkScreen* screen = gtk_widget_get_screen (widget);
-                gtk_icon_size_lookup_for_settings (
-                    gtk_settings_get_for_screen (screen),
-                    size, &width, &height);
-                pixbuf = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_for_screen (screen), icon,
-                    MAX (width, height), GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
-            }
-        }
-        if (!pixbuf)
-            pixbuf = gdk_pixbuf_new_from_file_at_size (icon_ready, 16, 16, NULL);
-    }
-    if (!pixbuf)
-        pixbuf = gtk_widget_render_icon (widget, GTK_STOCK_FIND, size, NULL);
-    return pixbuf;
-}
-
-gint
-sokoke_object_get_int (gpointer     object,
-                       const gchar* property)
-{
-    gint value = 0;
-
-    g_return_val_if_fail (object != NULL, FALSE);
-    g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
-    /* FIXME: Check value type */
-
-    g_object_get (object, property, &value, NULL);
-    return value;
-}
-
-gboolean
-sokoke_object_get_boolean (gpointer     object,
-                           const gchar* property)
-{
-    gboolean value = FALSE;
-
-    g_return_val_if_fail (object != NULL, FALSE);
-    g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
-    /* FIXME: Check value type */
-
-    g_object_get (object, property, &value, NULL);
-    return value;
-}
-
 /**
  * sokoke_action_create_popup_menu_item:
  * @action: a #GtkAction
@@ -679,3 +619,33 @@ sokoke_same_day (const time_t* day1,
     return same;
 }
 
+/**
+ * sokoke_days_between:
+ * @day1: a time_t timestamp value
+ * @day2: a time_t timestamp value
+ *
+ * Calculates the number of days between two timestamps.
+ *
+ * Return value: an integer.
+ **/
+gint
+sokoke_days_between (const time_t* day1,
+                     const time_t* day2)
+{
+    GDate* date1;
+    GDate* date2;
+    gint age;
+
+    date1 = g_date_new ();
+    date2 = g_date_new ();
+
+    g_date_set_time_t (date1, *day1);
+    g_date_set_time_t (date2, *day2);
+
+    age = g_date_days_between (date1, date2);
+
+    g_date_free (date1);
+    g_date_free (date2);
+
+    return age;
+}
