@@ -27,13 +27,6 @@ gchar*
 sokoke_magic_uri                    (const gchar*    uri,
                                      KatzeArray*     search_engines);
 
-void
-sokoke_entry_setup_completion       (GtkEntry*       entry);
-
-void
-sokoke_entry_append_completion      (GtkEntry*       entry,
-                                     const gchar*    text);
-
 typedef enum {
     SOKOKE_MENU_POSITION_CURSOR = 0,
     SOKOKE_MENU_POSITION_LEFT,
@@ -115,19 +108,6 @@ sokoke_widget_get_text_size          (GtkWidget*     widget,
                                       gint*          width,
                                       gint*          height);
 
-GdkPixbuf*
-sokoke_web_icon                      (const gchar*   icon,
-                                      GtkIconSize    size,
-                                      GtkWidget*     widget);
-
-gint
-sokoke_object_get_int                (gpointer       object,
-                                      const gchar*   property);
-
-gboolean
-sokoke_object_get_boolean            (gpointer       object,
-                                      const gchar*   property);
-
 GtkWidget*
 sokoke_action_create_popup_menu_item (GtkAction*     action);
 
@@ -143,4 +123,8 @@ gboolean
 sokoke_same_day                      (const time_t*  day1,
                                       const time_t*  day2);
 
+gint
+sokoke_days_between                  (const time_t*  day1,
+                                      const time_t*  day2);
+
 #endif /* !__SOKOKE_H__ */
